package hero.session;
/*
* 02/01/2002 - 15:24:07
*
* ProjectSessionEJB.java - 
* Copyright (C) 2002 Ecoo Team
* valdes@loria.fr
*
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

import hero.entity.EdgeState;
import hero.entity.NodeState;
import hero.interfaces.BnAgentEdgeLocal;
import hero.interfaces.BnAgentEdgeLocalHome;
import hero.interfaces.BnAgentEdgeValue;
import hero.interfaces.BnAgentLocal;
import hero.interfaces.BnAgentLocalHome;
import hero.interfaces.BnAgentValue;
import hero.interfaces.BnEdgeLocal;
import hero.interfaces.BnEdgeLocalHome;
import hero.interfaces.BnEdgeUtil;
import hero.interfaces.BnEdgeValue;
import hero.interfaces.BnNodeHookLocal;
import hero.interfaces.BnNodeHookLocalHome;
import hero.interfaces.BnNodeHookUtil;
import hero.interfaces.BnNodeInterHookLocal;
import hero.interfaces.BnNodeInterHookValue;
import hero.interfaces.BnNodeInterHookLocalHome;
import hero.interfaces.BnNodeInterHookUtil;
import hero.interfaces.BnNodeLightValue;
import hero.interfaces.BnNodeLocal;
import hero.interfaces.BnNodeLocalHome;
import hero.interfaces.BnNodePropertyLocal;
import hero.interfaces.BnNodePropertyValue;
import hero.interfaces.BnNodePropertyLocalHome;
import hero.interfaces.BnNodePropertyUtil;
import hero.interfaces.BnNodeUtil;
import hero.interfaces.BnNodeValue;
import hero.interfaces.BnProjectHookLocal;
import hero.interfaces.BnProjectHookLocalHome;
import hero.interfaces.BnProjectHookUtil;
import hero.interfaces.BnProjectInterHookLocal;
import hero.interfaces.BnProjectInterHookLocalHome;
import hero.interfaces.BnProjectInterHookUtil;
import hero.interfaces.BnProjectInterHookValue;
import hero.interfaces.BnProjectLocal;
import hero.interfaces.BnProjectLocalHome;
import hero.interfaces.BnProjectPropertyLocal;
import hero.interfaces.BnProjectPropertyLocalHome;
import hero.interfaces.BnProjectPropertyUtil;
import hero.interfaces.BnProjectPropertyValue;
import hero.interfaces.BnProjectValue;
import hero.interfaces.BnRoleLocal;
import hero.interfaces.BnRoleLocalHome;
import hero.interfaces.BnRoleUtil;
import hero.interfaces.BnRoleValue;
import hero.interfaces.BnUserLightValue;
import hero.interfaces.BnUserLocal;
import hero.interfaces.BnUserLocalHome;
import hero.interfaces.BnUserUtil;
import hero.interfaces.InvalidValueException;
import hero.interfaces.Constants;
import hero.util.DeleteEdgeException;
import hero.util.EventConstants;
import hero.util.HeroException;
import hero.util.NodeCancelledException;
import hero.util.NodeExecutingException;
import hero.util.NodeTerminatedException;
import hero.util.StrutsEdgeValue;
import hero.util.StrutsNodeValue;
import hero.util.TimerFilter;
import hero.interfaces.JMSServicesSessionUtil;
import hero.interfaces.JMSServicesSessionLocalHome;
import hero.interfaces.JMSServicesSessionLocal;

import hero.interfaces.BnRoleMapperLocal;
import hero.interfaces.BnRoleMapperLocalHome;
import hero.interfaces.BnRoleMapperUtil;

import hero.interfaces.BnNodePerformerAssignLocal;
import hero.interfaces.BnNodePerformerAssignLocalHome;
import hero.interfaces.BnNodePerformerAssignUtil;

import java.rmi.RemoteException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;
import java.util.Iterator;
import java.util.List;

import javax.ejb.CreateException;
import javax.ejb.FinderException;
import javax.ejb.RemoveException;
import javax.ejb.SessionBean;
import javax.ejb.SessionContext;
import javax.ejb.EJBException;
import javax.management.MBeanServer;
import javax.management.MBeanServerFactory;
import javax.management.NotificationFilter;
import javax.management.ObjectName;
import javax.naming.Context;

import org.apache.log4j.Logger;

/**
 *
 * Project Session Bean is an stateful session bean that provides the interface for workflow process
 * definition. By means of this API we can create workflow processes, project activities, transitions between activities ,
 * users and roles, properties, hooks, external agents...<br><br>
 * 
 * A Bonita Workflow process is composed by the activities (nodes) and the connections between these
 * activities (edges). For each project, the administrator/s set the list of responsibles to execute these
 * activites (users and roles). In the same way, the Project Session Bean API offers a great number of
 * functionalities for working with activities behaviors (hooks) and workflow relevant data (properties).<br><br>
 *  
 * This API can be used on different contexts depending on the needs of your application, so you can instantiate
 * an existing project in order to perform set, get, update and remove operations, you can create a new workflow 
 * project (by instantiating a project that does not exists), you can clone an existing project or creates a
 * new workflow instante of a project.   
 * 
 * <strong>The following lines shows a sample code to use this API in your application:<br><br></strong>
 * <br>
 * First of all you have to import the Project Session files:<br>
 * <br>
 * import hero.interfaces.ProjectSessionLocalHome;<br>
 * import hero.interfaces.ProjectSessionLocal;<br>
 * import hero.interfaces.ProjectSessionHome;<br>
 * import hero.interfaces.ProjectSession;<br>
 * import hero.interfaces.ProjectSessionUtil;<br>
 * <br>
 * Now, it is time to create the Project Session instance:<br>  
 * <br>
 * Like this if you want to use local interfaces:<br><br>
 * ProjectSessionLocalHome projecth = (ProjectSessionLocalHome)hero.interfaces.ProjectSessionUtil.getLocalHome();<br>
 * ProjectSessionLocal projectsession = projecth.create();<br>
 * <br>
 * or like this if you use remote interfaces:<br>  
 * <br>
 * ProjectSessionHome userh = (ProjectSessionHome)hero.interfaces.ProjectSessionUtil.getHome();<br>
 * ProjectSession projectsession = projecth.create();<br>
 * <br>
 * <br>
 * - If you want to use Project Session API for a specific project, you have to init this project by using:<br><br>
 * projectsession.initProject("yourProject");<br>
 * <br>
 * - In order to clone an existing project, you have to call the initProject method:<br><br>
 * projectsession.initProject("Project","CloneProject");<br><br>
 * 
 * - If you want to instantiate a project, you have to do the following:<br><br>
 * projectsession.instantiateProject("Project");<br><br>
 * 
 * Now you can use Project Session methods...  
 * 
 *
 * @ejb:bean name="ProjectSession"
 *           display-name="ProjectSession Bean"
 *           type="Stateful"
 *           transaction-type="Container"
 *           jndi-name="ejb/hero/ProjectSession"
 *           local-jndi-name="ejb/hero/ProjectSession_L"
 *
 * @ejb:ejb-ref	ejb-name="ProjectSession"
 *              	view-type="remote"
 *              	ref-name="ProjectSession"
 *
 * @ejb:ejb-ref ejb-name="BnProject"
 *              ref-name="myhero/BnProject"
 *
 * @ejb.permission role-name="Admin,user"
 * @jonas.bean
 *      ejb-name="ProjectSession"
 *      jndi-name="ejb/hero/ProjectSession"
 * 
 * 
 * @copyright INRIA
 * @author Miguel Valdes
 *
 **/

public class ProjectSessionBean implements SessionBean, EventConstants{

	// -------------------------------------------------------------------------
	// Static
	// -------------------------------------------------------------------------
	// Utility variable
	private static final Logger log =	Logger.getLogger(ProjectSessionBean.class);
	private static final Logger thelog = Logger.getLogger("log");

	// -------------------------------------------------------------------------
	// Members 
	// -------------------------------------------------------------------------

	private SessionContext mContext;
	
	private BnProjectLocalHome pHome;
	private BnProjectLocal mProject;
	private Collection subProjects = new ArrayList();
	private JMSServicesSessionLocal jms = null;
	private ArrayList visited = new ArrayList();

	// -------------------------------------------------------------------------
	// Methods
	// -------------------------------------------------------------------------

	// -------------------------------------------------------------------------
	// Add business methods
	// -------------------------------------------------------------------------

	/**
	 * Add a user to this project (This user must exist at bonita database)
	 *
	 * @param userName The name of the user to associate to the project
	 *
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 * @throws HeroException
	 *
	 **/
	public void addUser(String userName) throws HeroException {
		log.info(" Parameter: name=" + userName + " start by " + mContext.getCallerPrincipal().getName());
		BnUserLocalHome uHome;
		BnRoleLocalHome rHome;
		BnUserLocal mUser;
		try {
			uHome = hero.interfaces.BnUserUtil.getLocalHome();
			rHome = hero.interfaces.BnRoleUtil.getLocalHome();
		} catch (javax.naming.NamingException be) {
			log.error(be.getMessage());
			throw new HeroException(be.getMessage());
		}
		try {
			mUser = uHome.findByName(userName);
		} catch (javax.ejb.FinderException fe) {
			log.error("BnUser does not exist " + fe.getMessage());
			throw new HeroException("BnUser does not exist");
		}

		Collection cUsers = mProject.getBnUsers();
		cUsers.add(mUser);
		this.addRole(userName, "BnUser BnRole");
		this.setUserRole(userName, userName);
		jms.sendProjectEvent(ADDUSERPROJECT,this.getName(),userName);
		log.info("End");
	}

	/**
	 * Add a node to the project. This method creates a node with the corresponding
	 * node type and assign to it the hero.interfaces.Constants.InitialRole role.<br><br>
	 * 
	 * Possible node types are:<br>
	 * <br>
	 * 	hero.interfaces.Constants.Nd.AND_JOIN_NODE // Manual and join node <br>
	 *	hero.interfaces.Constants.Nd.OR_JOIN_NODE // Manual or join node <br>
	 *	hero.interfaces.Constants.Nd.AND_JOIN_AUTOMATIC_NODE // Automatic and join node <br>
	 *	hero.interfaces.Constants.Nd.OR_JOIN_AUTOMATIC_NODE // Automatic or join node <br>
	 *
	 * @param name The name of the node to add to the project
	 * @param nodeType The type of the node to add to the project 
	 *
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 * @throws HeroException
	 *
	 **/

	public void addNode(String name, int nodeType) throws HeroException {
		log.info("parameter: name=" + name + "nodeType=" + name +"  start by " + mContext.getCallerPrincipal().getName());
		BnNodeLocalHome nHome;
		BnNodeLocal mNode;

		try {
			nHome = hero.interfaces.BnNodeUtil.getLocalHome();
		} catch (javax.naming.NamingException be) {
			log.error(be.getMessage());
			throw new HeroException(be.getMessage());
		}		

		try {
			BnNodeLocal nf = nHome.findByName(name, mProject.getId());
			throw new HeroException("Node "+ name+ " already exist in project "+ mProject.getName());
		} catch (javax.ejb.FinderException nn) {
			log.error(nn.getMessage());
			try {
				BnNodeLocal nd = nHome.create(name, nodeType);
				BnNodeValue ndata = nd.getBnNodeValue();
				ndata.setCreationDate(new java.util.Date());
				nd.setBnNodeValue(ndata);
				Collection cNodes = mProject.getBnNodes();
				cNodes.add(nd);
				String creator = mContext.getCallerPrincipal().getName();
				BnUserLocalHome uHome =	hero.interfaces.BnUserUtil.getLocalHome();
				BnUserLocal user = uHome.findByName(creator);
				nd.setCreator(user);
				nd.setExecutor(user); // Only for JOnAS consistency
				BnRoleLocalHome rHome = BnRoleUtil.getLocalHome();
				BnRoleLocal role = rHome.findByName(hero.interfaces.Constants.INITIALROLE,mProject.getId());
				nd.setBnRole(role);

				if (nodeType == Constants.Nd.SUB_PROCESS_NODE)
				{
					try {
						BnProjectLocal pl = pHome.findByName(name);
						pl.setParent(mProject.getName());
					} catch (javax.ejb.FinderException pe) {
						log.error("Project "+ name+ " does not exist " + pe.getMessage());
						throw new HeroException("Project "+ name+ " does not exist");}
				}

				// Tell the world
				jms.sendNodeEvent(ADDNODE,this.getName(),name,nodeType,ndata.getState(),mContext.getCallerPrincipal().getName());
			} catch (InvalidValueException ie) {
				log.error(ie.getMessage());
				throw new HeroException(ie.getMessage());
			} catch (CreateException ce) {
				log.error(ce.getMessage());
				throw new HeroException(ce.getMessage());
			} catch (FinderException fe) {
				log.error(fe.getMessage());
				throw new HeroException(fe.getMessage());
			} catch (javax.naming.NamingException ne) {
				log.error(ne.getMessage());
				throw new HeroException(ne.getMessage());
			}
		}
		log.info("End");
	}
	
	/**
	 * Add a subProcess node to the project. This method creates the subProject from an existing
	 * project and creates the node associated to it. The type of created node is 
	 * hero.interfaces.Constants.Nd.SUB_PROCESS_NODE
	 *
	 * @param name The name of the node to add to the project
	 * @param projectName the name of the subProcess
	 *
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 * @throws HeroException
	 *
	 **/

	public void addNodeSubProcess(String name, String projectName) throws HeroException {
	log.info("parameters : name=" + name + "project=" + projectName +  " start by " + mContext.getCallerPrincipal().getName());
		try{
		  String aP = mProject.getName();
		  this.initProject(projectName,name);
		  Collection nodes = mProject.getBnNodes();
		  Iterator nds = nodes.iterator();
		  while (nds.hasNext()) // first activity will be initial
		  {
			BnNodeLocal subNode = (BnNodeLocal)nds.next();
			if (subNode.getState()==Constants.Nd.READY)
				this.setNodeState(subNode,Constants.Nd.INITIAL);
		  }
		  this.initProject(aP);
		  this.addNode(name,Constants.Nd.SUB_PROCESS_NODE);
		}catch (Exception ans){
			log.error("Error in addNodeSubProcess " + ans.getMessage());
			throw new HeroException("Error in addNodeSubProcess");
			}
	log.info("End");
	}

	/**
	 * Add an Agent to the Project
	 *
	 * @param name The name of the Agent to add to the project
	 *
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 * @throws HeroException
	 *
	 **/
	public void addAgent(String name) throws HeroException {
		log.info("parameter: name=" + name +  " start by " + mContext.getCallerPrincipal().getName());
		BnAgentLocalHome nHome;
		BnAgentLocal mAgent;
		try {
			nHome = hero.interfaces.BnAgentUtil.getLocalHome();
		} catch (javax.naming.NamingException be) {
			log.error(be.getMessage());
			throw new HeroException(be.getMessage());
		}

		try {
			BnAgentLocal nf = nHome.findByName(name, mProject.getId());
			throw new HeroException("BnAgent "+ name+ " already exist in project "+ mProject.getName());
		} catch (javax.ejb.FinderException nn) {
			log.error(nn.getMessage());

			try {
				BnAgentLocal ag = nHome.create(name);
				BnAgentValue adata = ag.getBnAgentValue();
				adata.setCreationDate(new java.util.Date());
				ag.setBnAgentValue(adata);
				Collection cAgents = mProject.getBnAgents();
				cAgents.add(ag);
				String creator = mContext.getCallerPrincipal().getName();
				BnUserLocalHome uHome =	hero.interfaces.BnUserUtil.getLocalHome();
				BnUserLocal user = uHome.findByName(creator);
				ag.setCreator(user);

				// Tell the world
				//		jms.sendNodeEvent(ADDNODE,this.getName(),name,nodeType,ndata.getState(),mContext.getCallerPrincipal().getName());
			} catch (InvalidValueException ie) {
				log.error(ie.getMessage());
				throw new HeroException(ie.getMessage());
			} catch (CreateException ce) {
				log.error(ce.getMessage());
				throw new HeroException(ce.getMessage());
			} catch (FinderException fe) {
				log.error(fe.getMessage());
				throw new HeroException(fe.getMessage());
			} catch (javax.naming.NamingException ne) {
				log.error(ne.getMessage());
				throw new HeroException(ne.getMessage());
			}
		}
		log.info("End");
	}

	/**
	 * Add an edge to the project. Creates a transition between two nodes by specifying
	 * the name of this edge. <strong> No more implemented </strong>
	 *
	 * @param name the name ot the edge
	 *
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 * @throws HeroException
	 *
	 **/
	public void addEdge(String name, String in, String out)
		throws HeroException {
		log.error("parameters: name=" + name + " in=" + in + " out=" + out +  " start by " + mContext.getCallerPrincipal().getName());
		throw new HeroException("no more implemented");
	}

	/**
	 * Add an edge between two nodes of the project [in] ---- [out]. This operation refresh
	 * the out node state. If new edge forms a cycle an exception is throwed.
	 *
	 * @param in the name of the in Node
	 * @param out the name of the out Node
	 *
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 * 
	 * @throws HeroException
	 *
	 **/
	public String addEdge(String in, String out) throws HeroException {
		log.info("parameters: in=" + in + " out:" + out +  " start by " + mContext.getCallerPrincipal().getName());
		BnEdgeLocalHome eHome;
		BnNodeLocalHome nHome;
		BnNodeLocal mInNode;
		BnNodeLocal mOutNode;
		BnEdgeLocal mEdge;
		try {
			nHome = hero.interfaces.BnNodeUtil.getLocalHome();
		} catch (javax.naming.NamingException be) {
			log.error(be.getMessage());
			throw new HeroException(be.getMessage());
		}
		try {
			mInNode = nHome.findByName(in, mProject.getId());
		} catch (FinderException fe) {
			log.error("In Node does not exist " + fe.getMessage());
			throw new HeroException("In Node does not exist");
		}
		try {
			mOutNode = nHome.findByName(out, mProject.getId());
		} catch (FinderException fe) {
			log.error("Out Node does not exist " + fe.getMessage());
			throw new HeroException("Out Node does not exist");
		}
		try {
			eHome = hero.interfaces.BnEdgeUtil.getLocalHome();
		} catch (javax.naming.NamingException be) {
			log.error(be.getMessage());
			throw new HeroException(be.getMessage());
		}
		visited.clear();
		if (!checkStructuralConstraints(in, out))
			throw new hero.util.EdgeCycleException("This edge add a cycle");
		 
		if (checkDynamics(out)) { // check state of Nodes
			if (checkDuplicateEdge(in, out)) {
				try {
					mEdge = eHome.create();
					mEdge.setBnProject(mProject);
					mEdge.setInBnNode(mInNode);
					mEdge.setOutBnNode(mOutNode);

					jms.sendEdgeEvent(ADDEDGE,this.getName(),mEdge.getName(),mInNode.getName(),mOutNode.getName(),mContext.getCallerPrincipal().getName());

				} catch (InvalidValueException ie) {
					log.error(ie.getMessage());
					throw new HeroException(ie.getMessage());
				} catch (CreateException ce) {
					log.error(ce.getMessage());
					throw new HeroException(ce.getMessage());
				}
			} else {
				throw new hero.util.DuplicatedEdgeException(
					"Cannot add edge. BnEdge between nodes already exist");
			}
		} else {
			throw new hero.util.NodeStartedException(
				"Cannot add edge. " + out + " is already started");
		}
		/*visited.clear();
		if (!checkStructuralConstraints(in, out))
			throw new hero.util.EdgeCycleException("This edge add a cycle");*/
		
		// update the current graph state
		int edgeState = hero.interfaces.Constants.edgeTransition[hero.interfaces.Constants.Ed.CONDITION_TRUE][mInNode.getState()];
		this.setEdgeState(mEdge, edgeState);
		//	mEdge.setState(edgeState);
		int newOperation = mOutNode.getActivation().state(mOutNode);
		int newState = mOutNode.getTransition().computeState(mOutNode, newOperation);
		if (newState == hero.interfaces.Constants.Nd.BAD_TRANSITION) {
			throw new HeroException("AddEdge generated a bad Node transition");
		}
		this.setNodeState(mOutNode, newState);
		//	jms.sendNodeEvent(READYINITIAL,this.getName(),nOut.getName(),nOut.getType(),INITIAL,context.getCallerPrincipal().getName());
		log.info("End");
		return mEdge.getName();
	}

	/**
	 * Add an AgentEdge to the Project [in] ---- [out]
	 *
	 * @param in the name of the in Agent
	 * @param out the name of the out Node
	 *
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 * 
	 * @throws HeroException
	 *
	 **/
	public String addAgentEdge(String in, String out) throws HeroException {
		log.info("parameters: in=" + in + " out:" + out +  " start by " + mContext.getCallerPrincipal().getName());
		BnAgentEdgeLocalHome eHome;
		BnNodeLocalHome nHome;
		BnAgentLocalHome aHome;
		BnAgentLocal mInAgent;
		BnNodeLocal mOutNode;
		BnAgentEdgeLocal mEdge;
		try {
			aHome = hero.interfaces.BnAgentUtil.getLocalHome();
		} catch (javax.naming.NamingException be) {
			log.error(be.getMessage());
			throw new HeroException(be.getMessage());
		}
		try {
			mInAgent = aHome.findByName(in, mProject.getId());
		} catch (FinderException fe) {
			log.error("In Agent does not exist " + fe.getMessage());
			throw new HeroException("In Agent does not exist");
		}
		try {
			nHome = hero.interfaces.BnNodeUtil.getLocalHome();
		} catch (javax.naming.NamingException be) {
			log.error(be.getMessage());
			throw new HeroException(be.getMessage());
		}
		try {
			mOutNode = nHome.findByName(out, mProject.getId());
		} catch (FinderException fe) {
			log.error("Out Node does not exist " + fe.getMessage());
			throw new HeroException("Out Node does not exist");
		}
		try {
			eHome = hero.interfaces.BnAgentEdgeUtil.getLocalHome();
		} catch (javax.naming.NamingException be) {
			log.error(be.getMessage());
			throw new HeroException(be.getMessage());
		}
		if (checkDynamics(out)) { // check state of Nodes
			if (checkDuplicateAgentEdge(in, out)) {
				try {
					mEdge = eHome.create();
					mEdge.setBnProject(mProject);
					mEdge.setInBnAgent(mInAgent);
					mEdge.setOutBnNode(mOutNode);

					//jms.sendEdgeEvent(ADDEDGE,this.getName(),mEdge.getName(),mInNode.getName(),mOutNode.getName(),mContext.getCallerPrincipal().getName());

				} catch (InvalidValueException ie) {
					log.error(ie.getMessage());
					throw new HeroException(ie.getMessage());
				} catch (CreateException ce) {
					log.error(ce.getMessage());
					throw new HeroException(ce.getMessage());
				}
			} else {
				throw new hero.util.DuplicatedEdgeException("Cannot add agentEdge. AgentEdge between nodes already exist");
			}
		} else {
			throw new hero.util.NodeStartedException(
				"Cannot add agentEdge. " + out + " is already started");
		}

		// update the current graph state
		this.setAgentEdgeState(mEdge, hero.interfaces.Constants.Ed.INITIAL);

		int newOperation = mOutNode.getActivation().state(mOutNode);
		int newState = mOutNode.getTransition().computeState(mOutNode, newOperation);
		if (newState == hero.interfaces.Constants.Nd.BAD_TRANSITION) {
			throw new HeroException("AddEdge generated a bad Node transition");
		}
		this.setNodeState(mOutNode, newState);
		//	jms.sendNodeEvent(READYINITIAL,this.getName(),nOut.getName(),nOut.getType(),INITIAL,context.getCallerPrincipal().getName());
		log.info("End");
		return mEdge.getName();
	}

	/**
	 * Add a role to the project. Creates a role within this project. The role is specific of this project
	 *
	 * @param name the name of the role
	 * @param description the description of the role
	 *
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 * 
	 * @throws HeroException
	 *
	 **/
	public void addRole(String roleName, String description)
		throws HeroException {
		BnRoleLocalHome roleh;
		log.info("parameters: roleName=" + roleName + " description=" + description +  " start by " + mContext.getCallerPrincipal().getName());
		try {
			roleh = BnRoleUtil.getLocalHome();
		} catch (javax.naming.NamingException ne) {
			log.error(ne.getMessage());
			throw new HeroException(ne.getMessage());
		}

		try {
			BnRoleLocal nf = roleh.findByName(roleName, mProject.getId());
			throw new HeroException("BnRole "+ roleName+ " already exist in project "+ mProject.getName());
		} catch (javax.ejb.FinderException nn) {
			log.error(nn.getMessage());

			BnRoleValue rv = new BnRoleValue();
			rv.setName(roleName);
			rv.setDescription(description);
			try {
				BnRoleLocal rl = roleh.create(rv);
				mProject.getBnRoles().add(rl);
				jms.sendRoleEvent(ADDROLE,this.getName(),roleName);
			} catch (javax.ejb.CreateException ce) {
				log.error(ce.getMessage());
				throw new HeroException(ce.getMessage());
			}
		}
		log.info("End");
	}
	
	/**
	 * Add a new iteration between two nodes. This methods sets an iteration which is stopped when
	 * the lastNode property "iterate" is false.  
	 *
	 * @param from the name of the first node
	 * @param to the name of the last node
	 *
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 * 
	 * @throws HeroException
	 *
	 **/
	public void addIteration(String from, String to) throws HeroException {
		log.info("parameters: from=" + from + " to=" + to +  " start by " + mContext.getCallerPrincipal().getName());
		if (!this.checkStructuralConstraints(from,to))
		{
		  this.setNodeProperty(from,"iterate","true",false);
		  this.setProperty("iterate_"+from+"-"+to,from);
		}
		else
			throw new HeroException("This iteration is not possible...");
		log.info("End");
	}

	/**
	 * Add hook to project. Add an existing hook file to the project. This type of hook use a 
	 * Java or TCL file loaded at run time. <strong> This functionality is disabled. Use {@link #addInterHook(String,String,int,String) addInterHook}
	 * method to add a beanshell hook to the project. </strong>
	 *
	 * @param hookName the name of the hook
	 * @param eventName the event
	 * @param hookType the type
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 * @throws HeroException
	 *
	 **/
	public void addHook(String hookName, String eventName, int hookType)
		throws HeroException {
		log.info("parameters: hookName=" + hookName + " eventName=" + eventName +  " start by " + mContext.getCallerPrincipal().getName());
		BnProjectHookLocalHome pHome;
		BnProjectHookLocal ppLocal;

		try {
			pHome = BnProjectHookUtil.getLocalHome();
		} catch (javax.naming.NamingException ne) {
			log.error(ne.getMessage());
			throw new HeroException(ne.getMessage());
		}

		try {
			BnProjectHookLocal pf = pHome.findByName(mProject.getId(), hookName);
			throw new HeroException("BnProject hook "+ hookName	+ " already exist in project "+ mProject.getName());
		} catch (javax.ejb.FinderException pn) {
			log.error(pn.getMessage());
			try {
				BnProjectHookLocal h =	pHome.create(hookName, eventName, hookType);
				h.setBnProject(mProject);
			} catch (CreateException fe) {
				log.error(fe.getMessage());
				throw new HeroException(fe.getMessage());
			} catch (InvalidValueException ie) {
				log.error(ie.getMessage());
				throw new HeroException(ie.getMessage());
			}
		}
		log.info("End");
	}

	/**
	 * Add hook to a node. Add an existing hook file to the node. This type of hook use a 
	 * Java or TCL file loaded at run time. <strong> This functionality is disabled.
	 * Use {@link #addNodeInterHook(String,String,String,int,String) addNodeInterHook}
	 * method to add a beanshell hook to the node. </strong>
	 *
	 * @param nodeName the name of the node
	 * @param hookName the name of the hook
	 * @param eventName the name of the event
	 * @param hookType the type of the hook
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 * @throws HeroException
	 *
	 **/
	public void addNodeHook(String nodeName,String hookName,String eventName,int hookType)	throws HeroException {
		log.info("parameters: node="+ nodeName+ " hookName="+ hookName+ " eventName="	+ eventName +  " start by " + mContext.getCallerPrincipal().getName());
		BnNodeHookLocalHome pHome;
		BnNodeHookLocal ppLocal;

		BnNodeLocal nd = mProject.getBnNode(nodeName);
		try {
			pHome = BnNodeHookUtil.getLocalHome();
		} catch (javax.naming.NamingException ne) {
			log.error(ne.getMessage());
			throw new HeroException(ne.getMessage());
		}

		try {
			BnNodeHookLocal nf = pHome.findByName(nd.getId(), hookName);
			throw new HeroException("BnNode hook "+ hookName+ " already exist in node "+ nodeName);
		} catch (javax.ejb.FinderException nn) {
			log.error(nn.getMessage());
			try {
				BnNodeHookLocal h = pHome.create(hookName, eventName, hookType);
				h.setBnNode(nd);
			} catch (CreateException fe) {
				log.error(fe.getMessage());
				throw new HeroException(fe.getMessage());
			} catch (InvalidValueException ie) {
				log.error(ie.getMessage());
				throw new HeroException(ie.getMessage());
			}
		}
		log.info("End");
	}
	
	/**
	 * Add mapper to a role. Add an existing mapper to the role. This type of mapper use a 
	 * Java  file loaded at run time.
	 *
	 * @param roleName the name of the role
	 * @param mapperName the name of the mapper
	 * @param mapperType the type of the mapper
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 * @throws HeroException
	 *
	 **/
	public void addRoleMapper(String roleName,String mapperName,int mapperType)	throws HeroException {
		// logs trace
		log.info("parameters: roleName="+ roleName+ " mapperName="+ mapperName+ " mapperType="+ mapperType+ " start by " + mContext.getCallerPrincipal().getName());
		
		BnRoleMapperLocalHome mapperh;
		BnRoleLocalHome roleh;
		BnRoleMapperLocal ppLocal = null;
		BnRoleLocal role = null;
   
		try {
			 mapperh = BnRoleMapperUtil.getLocalHome();
			 roleh = BnRoleUtil.getLocalHome();
		} catch (javax.naming.NamingException ne) {
			log.error(ne.getMessage());
			throw new HeroException(ne.getMessage());
		} 
		// Get objet role local with findByName from BnRoleBean
		try {
		 role = roleh.findByName(roleName, mProject.getId());
		} catch (FinderException pfe) {
		  log.error(pfe.getMessage());
		  throw new HeroException("The bnRole does not exist for role : " + roleName);
		}	    
		// if the role has not already a mapper put one
		ppLocal = role.getBnRoleMapper();
		 if (ppLocal != null){
			throw new HeroException("BnRoleMapper mapper  " + role.getBnRoleMapper().getName() + " already exists for role "  + role.getName() + " within project Id : " + mProject.getId());		 	
		 }
		else {
			try {
				ppLocal = mapperh.create(mapperName, mapperType);
				ppLocal.setBnRole(role);			
			} catch (CreateException fe) {
				log.error(fe.getMessage());
				throw new HeroException(fe.getMessage());
			} catch (InvalidValueException ie) {
				log.error(ie.getMessage());
				throw new HeroException(ie.getMessage());
			}
		}
	log.info("End");
	}
	
	/**
	* Add performerAssign to a node. Add an existing performerAssign to the node. This type of performerAssign use a 
	* Java  file loaded at run time.
	*
	* @param nodeName the name of the node
	* @param performerAssignName the name of the performerAssign
	* @param performerAssignType the type of the performerAssign
	* @ejb:interface-method view-type="both"
	* @ejb:transaction type="Required"
	* @throws HeroException
	*
	**/
   public void addNodePerformerAssign(String nodeName,String performerAssignName,int performerAssignType, String propertyName)	throws HeroException {
	   // logs trace
	   log.debug("addNodePerformerAssign: node = "+ nodeName+ " performerAssignName = "+ performerAssignName);
		
	   BnNodePerformerAssignLocalHome performerAssignh;
	   BnNodeLocalHome nodeh;
	   BnNodePerformerAssignLocal ppLocal = null;
	   BnNodeLocal node = null;
   
	   try {
			performerAssignh = BnNodePerformerAssignUtil.getLocalHome();
			nodeh = BnNodeUtil.getLocalHome();
	   } catch (javax.naming.NamingException ne) {
		   throw new HeroException(ne.getMessage());
	   } 
	   // Get objet node local with findByName from BnNodeBean
	   try {
		node = nodeh.findByName(nodeName, mProject.getId());
	   } catch (FinderException pfe) {
		 throw new HeroException("The bnNode does not exist for node : " + nodeName);
	   }	    
	   // if the node has not already a performerAssign put one
	   ppLocal = node.getBnNodePerformerAssign();
		if (ppLocal != null){
		   throw new HeroException("BnNodePerformerAssign performerAssign  " + node.getBnNodePerformerAssign().getName() + " already exists for node "  + node.getName() + " within project Id : " + mProject.getId());		 	
		}
	   else {
		   try {
			   ppLocal = performerAssignh.create(performerAssignName, performerAssignType, propertyName);
			   ppLocal.setBnNode(node);			
		   } catch (CreateException fe) {
			   throw new HeroException(fe.getMessage());
		   } catch (InvalidValueException ie) {
			   throw new HeroException(ie.getMessage());
		   }
	   }
   }
	
	/**
	 * Add interhook to a node. Set a new hook to this node by setting the eventName, hookType and
	 * script values.<br><br>
	 * 
	 * The possible values of the argumets are:<br><br>
	 * <strong>eventName:<br></strong>
	 * 	hero.interfaces.Constants.Nd.BEFORESTART<br>
	 *  hero.interfaces.Constants.Nd.AFTERSTART<br>
	 *	hero.interfaces.Constants.Nd.BEFORETERMINATE<br>
	 *	hero.interfaces.Constants.Nd.AFTERTERMINATE<br>
	 *	hero.interfaces.Constants.Nd.ONCANCEL<br>
	 *	hero.interfaces.Constants.Nd.ANTICIPATE<br><br>
	 *
	 * <strong>hookType:<br></strong>
	 * 
	 *  hero.hook.Hook.BSINTERACTIVE<br><br>
	 * 
	 * <strong>script:<br></strong>
	 * The beanshell source code associated to this hook.<br><br>
	 * 
	 * <strong>The following sample code illustrates how to use this method:<br><br></strong>
	 *
	 *  ...<br>
	 *  String script = "import hero.interfaces.*;\n"<br>
	 * 		            + "import hero.interfaces.BnNodeLocal;\n"<br>
	 *			        + "beforeStart (Object b,Object n) {\n\n\n"<br>
	 *					+ "System.out.println(\"Hook Test\");\n"<br>
	 *			        + "}";<br>
	 *  <strong>yourProjectSessionInstance</strong>.addNodeInterHook("NodeName","Hook_Name",Constants.Nd.BEFORESTART,hero.hook.Hook.BSINTERACTIVE,script);
	 *
	 * @param nodeName the name of the node
	 * @param hookName the name of the hook
	 * @param eventName the name of the event
	 * @param hookType the type of the hook
	 * @param script the value of the script 
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 * @throws HeroException
	 *
	 **/
	public void addNodeInterHook(String nodeName,String hookName,String eventName,int hookType,String value)
		throws HeroException {
		log.info("parameters: node="+ nodeName+ " hookName="+ hookName+ " eventName="+ eventName+ " value="+ value +  " start by " + mContext.getCallerPrincipal().getName());
		BnNodeInterHookLocalHome pHome;
		BnNodeInterHookLocal ppLocal;

		BnNodeLocal nd = mProject.getBnNode(nodeName);
		try {
			pHome = BnNodeInterHookUtil.getLocalHome();
		} catch (javax.naming.NamingException ne) {
			log.error(ne.getMessage());
			throw new HeroException(ne.getMessage());
		}

		try {
			BnNodeInterHookLocal nf = pHome.findByName(nd.getId(), hookName);
			throw new HeroException("Interactive node hook "+ hookName+ " already exist in node "+ nodeName);
		} catch (javax.ejb.FinderException nn) {
			log.error(nn.getMessage());
			try {
				BnNodeInterHookLocal h = pHome.create(hookName, eventName, hookType, value);
				h.setBnNode(nd);
			} catch (CreateException fe) {
				log.error(fe.getMessage());
				throw new HeroException(fe.getMessage());
			} catch (InvalidValueException ie) {
				log.error(ie.getMessage());
				throw new HeroException(ie.getMessage());
			}
		}
		log.info("End");
	}

	/**
	 * Add interhook to the project. Creates new hook associated to all project activites.
	 * See the {@link #addNodeInterHook(String,String,String,int,String) addNodeInterHook} method in order
	 * to identify arguments values.
	 *
	 * @param hookName the name of the hook
	 * @param eventName the name of the event
	 * @param hookType the type of the hook
	 * @param script the value of the script 
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 * @throws HeroException
	 *
	 **/
	public void addInterHook(String hookName,String eventName,int hookType,String value)
		throws HeroException {
		log.info("parameters: hookName="+ hookName+ " eventName="+ eventName+ " value="+ value +  " start by " + mContext.getCallerPrincipal().getName());
		BnProjectInterHookLocalHome pHome;
		BnProjectInterHookLocal ppLocal;

		try {
			pHome = BnProjectInterHookUtil.getLocalHome();
		} catch (javax.naming.NamingException ne) {
			log.error(ne.getMessage());
			throw new HeroException(ne.getMessage());
		}

		try {
			BnProjectInterHookLocal nf = pHome.findByName(mProject.getId(), hookName);
			throw new HeroException("Interactive hook "+ hookName+ " already exist in project "+ mProject.getName());
		} catch (javax.ejb.FinderException nn) {
			log.error(nn.getMessage());
			try {
				BnProjectInterHookLocal h = pHome.create(hookName, eventName, hookType, value);
				h.setBnProject(mProject);
			} catch (CreateException fe) {
				log.error(fe.getMessage());
				throw new HeroException(fe.getMessage());
			} catch (InvalidValueException ie) {
				log.error(ie.getMessage());
				throw new HeroException(ie.getMessage());
			}
		}
		log.info("End");
	}

	// -------------------------------------------------------------------------
	// Delete business methods
	// -------------------------------------------------------------------------

	/**
	 * Delete a node from the project. If this node is in executing, terminated or cancelled state, the
	 * method throws an execption
	 *
	 * @param name the name ot the edge
	 *
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 * @throws HeroException
	 *
	 **/
	public void deleteNode(String name) throws HeroException {
		// check deletion conditions
		log.info("parameter: name=" + name +  " start by " + mContext.getCallerPrincipal().getName());
		 
		BnNodeLocal nd = mProject.getBnNode(name);
		if (nd.isExecuting()) {
			throw new NodeExecutingException("Cannot delete node. It is executing");
		}
		if (nd.isTerminated()) {
			throw new NodeTerminatedException("Cannot delete node. It is terminated");
		}
		if (nd.isCancelled()) {
			throw new NodeCancelledException("Cannot delete node. It is cancelled");
		}
		
		if (this.getNodeType(name)==Constants.Nd.SUB_PROCESS_NODE)
		{
		   try {
			  BnProjectLocal plocal = pHome.findByName(name);
			  Collection nodes = plocal.getBnNodes();
			  Iterator nds = nodes.iterator();
			  while (nds.hasNext())            // Verifies if the project has any subProcesses
			  {
				 BnNodeLocal nod = (BnNodeLocal)nds.next();
				 if (nod.getType()==Constants.Nd.SUB_PROCESS_NODE)
				   throw new HeroException("Error in delete project: You have to delete subProcesses first !!");
			  }
			  plocal.remove();
			  jms.sendProjectEvent(DELETEPROJECT, name);
			} catch (RemoveException pre) {
			  log.error("Cannot remove project " + pre.getMessage());
			  throw new HeroException("Cannot remove project " + pre.getMessage());
			} catch (FinderException pfe) {
			  log.error("Cannot find project " + pfe.getMessage());
			  throw new HeroException("Cannot find project " + pfe.getMessage());
			}
		}
		
		Collection edgesIn = nd.getInBnEdges();
		Object[] edai = edgesIn.toArray();
		for (int i = 0; i < edai.length; i++) {
			edgeDeletion((BnEdgeLocal) edai[i]);
		}
		Collection edgesOut = nd.getOutBnEdges();
		Object[] edao = edgesOut.toArray();
		for (int i = 0; i < edao.length; i++) {
			edgeDeletion((BnEdgeLocal) edao[i]);
		}
		try {
			int state = nd.getState();
			int type = nd.getType();
			nd.remove();
			jms.sendNodeEvent(DELETENODE,this.getName(),name,type,state,mContext.getCallerPrincipal().getName());
			removeOldNotification(name + ":" + this.getName());
		} catch (RemoveException re) {
			log.error("Cannot remove node " + re.getMessage());
			throw new HeroException("Cannot remove node " + re.getMessage());
		}
		log.info("End");
	}

	/**
	 * Delete an edge to the project. If out node of this edge is in executing, terminated or cancelled state, the
	 * method throws an execption.
	 *
	 * @param name the name ot the edge
	 *
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 * @throws HeroException
	 *
	 **/
	public void deleteEdge(String name) throws HeroException {
		// check deletion conditions
		log.info("parameter: name=" + name +  " start by " + mContext.getCallerPrincipal().getName());
		BnEdgeLocalHome eh;
		BnEdgeLocal ed;
		try {
			eh = BnEdgeUtil.getLocalHome();
		} catch (javax.naming.NamingException ne) {
			log.error(ne.getMessage());
			throw new HeroException(ne.getMessage());
		}
		try {
			ed = eh.findByName(name, mProject.getId());
		} catch (FinderException fe) {
			log.error("edge does not exists " + fe.getMessage());
			throw new HeroException("edge does not exists " + name);
		}
		String nodeIn = ed.getInBnNode().getName();
		String nodeOut = ed.getOutBnNode().getName();
		edgeDeletion(ed);
		jms.sendEdgeEvent(DELETEEDGE,this.getName(),name,nodeIn,nodeOut,mContext.getCallerPrincipal().getName());
		log.info("End");
	}

	/** Actually deletes an edge and restore the state of the outgoing node
	 *  <strong>Not a business method</strong>
	**/

	public void edgeDeletion(BnEdgeLocal ed) throws HeroException {
		BnNodeLocal nd = ed.getOutBnNode();
		log.info(" start by " + mContext.getCallerPrincipal().getName());
		if (nd.isExecuting()) {
			throw new DeleteEdgeException("Cannot delete edge. Out node executing");
		}
		if (nd.isTerminated()) {
			throw new DeleteEdgeException("Cannot delete edge. Out node terminated");
		}
		if (nd.isCancelled()) {
			throw new DeleteEdgeException("Cannot delete edge. Out node cancelled");
		}
		try {
			ed.remove();
			int newOperation = nd.getActivation().state(nd);
			int newState = nd.getTransition().computeState(nd, newOperation);
			if (newState == hero.interfaces.Constants.Nd.BAD_TRANSITION) {
				throw new HeroException("AddEdge generated a bad BnNode transition");
			}
			this.setNodeState(nd, newState);
		} catch (RemoveException e) {
			log.error("deleteNode failed " + e.getMessage());
			throw new HeroException("deleteNode failed " + e.getMessage());
		}
		log.info("End");
	}


	/**
	 * Detete a hook.<strong> This functionality is disabled.
	 * Use {@link #deleteInterHook(String) deleteInterHook} 
	 * method to delete a hook of this project. </strong>
	 *
	 * @param hookName the name of the hook
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 * @throws HeroException
	 *
	 **/
	public void deleteHook(String hookName) throws HeroException {
	log.info(" start by " + mContext.getCallerPrincipal().getName());
		BnProjectHookLocalHome pHome;
		BnProjectHookLocal ppLocal;

		try {
			pHome = BnProjectHookUtil.getLocalHome();
		} catch (javax.naming.NamingException ne) {
			log.error(ne.getMessage());
			throw new HeroException(ne.getMessage());
		}

		try {
			BnProjectHookLocal ph =	pHome.findByName(mProject.getId(), hookName);
			ph.remove();
		} catch (FinderException fe) {
			log.error(fe.getMessage());
			throw new HeroException(fe.getMessage());
		} catch (RemoveException rm) {
			log.error(rm.getMessage());
			throw new HeroException(rm.getMessage());
		}
		log.info("End");
	}

	/**
	 * Delete a node hook.<strong> This functionality is disabled.
	 * Use {@link #deleteNodeInterHook(String,String) deleteNodeInterHook}
	 * method to delete a hook of this node. </strong>
	 *
	 * @param nodeName the name of the node
	 * @param hookName the name 
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 * @throws HeroException
	 *
	 **/
	public void deleteNodeHook(String nodeName, String hookName)
		throws HeroException {
		log.info("parameters: nodename=" + nodeName + "Hookname=" + hookName + " start by " + mContext.getCallerPrincipal().getName());
		BnNodeHookLocalHome pHome;
		BnNodeHookLocal ppLocal;
		BnNodeLocal nd = mProject.getBnNode(nodeName);
		try {
			pHome = BnNodeHookUtil.getLocalHome();
		} catch (javax.naming.NamingException ne) {
			log.error(ne.getMessage());
			throw new HeroException(ne.getMessage());
		}
		try {
			BnNodeHookLocal nh = pHome.findByName(nd.getId(), hookName);
			nh.remove();
		} catch (FinderException fe) {
			log.error(fe.getMessage());
			throw new HeroException(fe.getMessage());
		} catch (RemoveException rm) {
			log.error(rm.getMessage());
			throw new HeroException(rm.getMessage());
		}
		log.info("End");
	}

	/**
	  * Delete a node interHook.
	  *
	  * @param nodeName the name of the node
	  * @param hookName the name of the interHook
	  * @ejb:interface-method view-type="both"
	  * @ejb:transaction type="Required"
	  * @throws HeroException
	  *
	  **/
	public void deleteNodeInterHook(String nodeName, String interHookName)
		throws HeroException {
		log.info("parameters: nodename=" + nodeName + "InerhookName="  + interHookName  +  " start by " + mContext.getCallerPrincipal().getName());
		BnNodeInterHookLocalHome pHome;
		BnNodeInterHookLocal ppLocal;
		BnNodeLocal nd = mProject.getBnNode(nodeName);
		try {
			pHome = BnNodeInterHookUtil.getLocalHome();
		} catch (javax.naming.NamingException ne) {
			log.error(ne.getMessage());
			throw new HeroException(ne.getMessage());
		}
		try {
			BnNodeInterHookLocal nh =
				pHome.findByName(nd.getId(), interHookName);
			nh.remove();
		} catch (FinderException fe) {
			log.error(fe.getMessage());
			throw new HeroException(fe.getMessage());
		} catch (RemoveException rm) {
			log.error(rm.getMessage());
			throw new HeroException(rm.getMessage());
		}
		log.info("End");
	}

	/**
	  * Detete a interHook of this project. This method deletes this hook from all projects nodes.
	  *
	  * @param hookName the name of the hook
	  * @ejb:interface-method view-type="both"
	  * @ejb:transaction type="Required"
	  * @throws HeroException
	  *
	  **/
	public void deleteInterHook(String interHookName) throws HeroException {
		BnProjectInterHookLocalHome pHome;
		BnProjectInterHookLocal ppLocal;
		log.info("parameter: interhookName=" + interHookName + " start by " + mContext.getCallerPrincipal().getName());
		try {
			pHome = BnProjectInterHookUtil.getLocalHome();
		} catch (javax.naming.NamingException ne) {
			log.error(ne.getMessage());
			throw new HeroException(ne.getMessage());
		}

		try {
			BnProjectInterHookLocal ph =
				pHome.findByName(mProject.getId(), interHookName);
			ph.remove();
		} catch (FinderException fe) {
			log.error(fe.getMessage());
			throw new HeroException(fe.getMessage());
		} catch (RemoveException rm) {
			log.error(rm.getMessage());
			throw new HeroException(rm.getMessage());
		}
		log.info("End");
	}
	
	/**
	 * Delete a role mapper.
	 *
	 * @param roleName the name of the node
	 * @param roleName name 
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 * @throws HeroException
	 *
	 **/
	public void deleteRoleMapper(String roleName) throws HeroException {
		log.info("parameter: roleName=" + roleName + " start by " + mContext.getCallerPrincipal().getName());
		BnRoleLocalHome pHome;
		BnRoleMapperLocal mapperL;
		BnRoleLocal roleL;
		try {
			pHome = BnRoleUtil.getLocalHome();
		} catch (javax.naming.NamingException ne) {
			log.error(ne.getMessage());
			throw new HeroException(ne.getMessage());
		}
				
		try {
			roleL = pHome.findByName(roleName, mProject.getId());
			mapperL = roleL.getBnRoleMapper();
		} catch (FinderException fe) {
			log.error(fe.getMessage());
			throw new HeroException(fe.getMessage());				
		}
		
		try {
			mapperL.remove();
		} catch (RemoveException rm) {
			log.error(rm.getMessage());
			throw new HeroException(rm.getMessage());
		}
	log.info("End");
	}

	/**
	 * Delete a property of the project. Deletes the project property associated to this key.
	 *
	 * @param key the key of the property
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 * @throws HeroException
	 *
	 **/
	public void deleteProperty(String key) throws HeroException {
		log.info("parameter: key=" + key + " start by " + mContext.getCallerPrincipal().getName());

		BnProjectPropertyLocalHome pHome;
		BnProjectPropertyLocal ppLocal;

		try {
			pHome = BnProjectPropertyUtil.getLocalHome();
		} catch (javax.naming.NamingException ne) {
			log.error(ne.getMessage());
			throw new HeroException(ne.getMessage());
		}
		Collection c = null;
		try {
			c = pHome.findByTheKey(mProject.getId(), key);
		} catch (FinderException fe) {
			log.error(fe.getMessage());
			throw new HeroException(fe.getMessage());
		}

		if (!c.isEmpty()) {
			try {
				ppLocal = (BnProjectPropertyLocal) (c.toArray())[0];
				ppLocal.remove();
			} catch (RemoveException rm) {
				log.error(rm.getMessage());
				throw new HeroException(rm.getMessage());
			}

		} else {
			throw new HeroException("Property not found :" + key);
		}
		log.info("End");
	}

	/**
	 * Delete a property of a node. Deletes the node property associated to this key 
	 *
	 * @param nodeName the name of the node
	 * @param key the key of the property
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 * @throws HeroException
	 *
	 **/
	public void deleteNodeProperty(String nodeName, String key)
		throws HeroException {
		log.info("parameters: node=" + nodeName + " key=" + key + " start by " + mContext.getCallerPrincipal().getName());

		BnNodePropertyLocalHome pHome;
		BnNodePropertyLocal propertyLocal = null;

		BnNodeLocal nd = mProject.getBnNode(nodeName);

		try {
			pHome = BnNodePropertyUtil.getLocalHome();
		} catch (javax.naming.NamingException ne) {
			log.error(ne.getMessage());
			throw new HeroException(ne.getMessage());
		}
		Collection c = null;
		try {
			c = pHome.findByTheKey(nd.getId(), key);
		} catch (FinderException fe) {
			log.error(fe.getMessage());
			throw new HeroException(fe.getMessage());
		}

		if (!c.isEmpty()) {
			try {
				propertyLocal = (BnNodePropertyLocal) (c.toArray())[0];
				propertyLocal.remove();
			} catch (RemoveException rm) {
				log.error(rm.getMessage());
				throw new HeroException(rm.getMessage());
			}

		} else {
			throw new HeroException("Property not found :" + key);
		}
		log.info("End");
	}

	// -------------------------------------------------------------------------
	// Set business methods
	// -------------------------------------------------------------------------

	/**
	 * Set a role to this user. This role is assigned to the user in the context of
	 * the actual project.
	 *
	 * @param userName the name of the user
	 * @param roleName the name of the role
	 *
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 * @throws HeroException
	 *
	 **/
	public void setUserRole(String userName, String roleName)
		throws HeroException {
		log.info("parameters: user=" + userName + " role=" + roleName + " start by " + mContext.getCallerPrincipal().getName());
		BnRoleLocalHome roleh;
		BnUserLocalHome userh;
		try {
			roleh = BnRoleUtil.getLocalHome();
			userh = BnUserUtil.getLocalHome();
		} catch (javax.naming.NamingException ne) {
			log.error(ne.getMessage());
			throw new HeroException(ne.getMessage());
		}
		try {
			/*NamingManagementLocal namingManagementLocal = NamingManagementUtil.getLocalHome().create();			
			BnUserLocal user = userh.findByName(namingManagementLocal.getUsersRootName() + namingManagementLocal.getSeparator()+userName);*/
			BnUserLocal user = userh.findByName(userName);
			BnRoleLocal role = roleh.findByName(roleName, mProject.getId());
			user.getBnRoles().add(role);
			jms.sendUserRoleEvent(SETUSERROLE,this.getName(),userName,roleName);
		} catch (Exception fe) {
			log.error(fe.getMessage());
			throw new HeroException(fe.getMessage());
		}
		log.info("End");
	}

	/**
	 * Propagate node properties. Propagates all node properties to connected nodes. 
	 * @param nodeName the name of the Node
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 * @throws HeroException
	 *
	 **/
	public void propagateNodeProperties(String nodeName) throws HeroException {
		log.info("parameter: name=" + nodeName + " start by " + mContext.getCallerPrincipal().getName());

		BnNodePropertyLocalHome pHome;
		BnNodePropertyLocal propertyLocal = null;

		BnNodeLocal nd = mProject.getBnNode(nodeName);

		Collection c = nd.getBnProperties();
		Collection c2 = nd.getOutBnEdges();
		if ((!c.isEmpty()) && (!c2.isEmpty())) {
			BnNodePropertyLocal p;
			BnEdgeLocal ed;
			for (Iterator i = c.iterator(); i.hasNext();) {
				p = (BnNodePropertyLocal) i.next();
				if (p.getPropagate() && !p.getTheKey().equals("iterate")) {
					for (Iterator i2 = c2.iterator(); i2.hasNext();) {
						ed = (BnEdgeLocal) i2.next();
						this.setNodeProperty((ed.getOutBnNode()).getName(),p.getTheKey(),p.getTheValue());
					}
				}
			}
		}
		log.info("End");
	}

	/**
	 * Sets the role of an activity. Changes the role of this activity
	 *
	 * @param name the name of the activity
	 * @param role the name of the role
	 *
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 * @throws HeroException
	 *
	 **/
	public void setNodeRole(String name, String role) throws HeroException {
		log.info("parameters: name=" + name + " role=" + role + " start by " + mContext.getCallerPrincipal().getName());
		BnRoleLocalHome rh;
		BnNodeLocalHome nh;
		BnNodeLocal nl;
		BnRoleLocal rl;
		try {
			rh = BnRoleUtil.getLocalHome();
			nh = BnNodeUtil.getLocalHome();
		} catch (javax.naming.NamingException ne) {
			log.error(ne.getMessage());
			throw new HeroException(ne.getMessage());
		}
		try {
			nl = nh.findByName(name, mProject.getId());
		} catch (FinderException fe) {
			log.error("BnNode does not exists " + fe.getMessage());
			throw new HeroException("BnNode does not exists " + name);
		}
		try {
			rl = rh.findByName(role, mProject.getId());
		} catch (FinderException fe) {
			log.error("BnRole does not exists " + fe.getMessage());
			throw new HeroException("BnRole does not exists " + role);
		}
		nl.setBnRole(rl);
		jms.sendProjectEvent(CHANGEROLE, this.getName());
		log.info("End");
	}
	
	/**
	 * Set the node state. Set start date of this node when starts the execution and the end date when node
	 * is terminated.
	 *
	 * @param name name of the node
	 * @param state new state of the node
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 * @throws HeroException
	 *
	 **/
	public void setNodeState(BnNodeLocal node, int state)	throws HeroException {
			log.info("parameters: name=" + node.getName() + " state=" + state + " start by " + mContext.getCallerPrincipal().getName());
			node.setState(state);
			if (state == hero.interfaces.Constants.Nd.EXECUTING) {
				node.setStartDate(new java.util.Date());
			}
			if (state == hero.interfaces.Constants.Nd.TERMINATED) {
				node.setEndDate(new java.util.Date());
				this.propagateNodeProperties(node.getName());
			}
			jms.sendNodeEvent(SETNODESTATE,this.getName(),node.getName(),node.getType(),state,mContext.getCallerPrincipal().getName());
			log.info("End");
		}

	/**
	 * Set all initial roles to ready if node does not has in edges.
	 *
	 * @ejb:interface-method view-type="local"
	 * @ejb:transaction type="Required"
	 * @throws HeroException
	 *
	 **/
	public void setNodesReady() throws HeroException {
	log.info(" start by " + mContext.getCallerPrincipal().getName());
		try{
		   Collection nodes = mProject.getBnNodes();
		  Iterator nds = nodes.iterator();
		  while (nds.hasNext()) // first activity will be initial
		  {
			BnNodeLocal subNode = (BnNodeLocal)nds.next();
			if (this.getNodeInEdges(subNode.getName()).size()==0 && subNode.getState()==Constants.Nd.INITIAL)
				this.setNodeState(subNode,Constants.Nd.READY);
		  }
		}catch (Exception ans){
			log.error("Error in setNodeReady " + ans.getMessage());
			throw new HeroException("Error in setNodeReady");
		}
		log.info("End");
	}
	
	/**
	 * Propagate subProcess activity properties to subProcess
	 *
	 * @param node the name of the node
	 * @ejb:interface-method view-type="local"
	 * @ejb:transaction type="Required"
	 * @throws HeroException
	 *
	 **/

	public void propagatesParentProperties(String node) throws HeroException {
		log.info("parameter: nodeName=" + node + " start by " + mContext.getCallerPrincipal().getName());
		BnProjectLocal pl = mProject;
		try{
		  Collection props = mProject.getBnNode(node).getBnProperties();
		  this.initProject(node);
		  Iterator i = props.iterator();
		  while (i.hasNext())
		  {
			BnNodePropertyLocal prop = (BnNodePropertyLocal)i.next();											
			this.setProperty(prop.getTheKey(),prop.getTheValue());
		  }
		  this.setNodesReady();
		  mProject=pl;																		
		}catch (Exception ans){
			mProject=pl;
			log.error("Error in propagatesParentProperties " + ans.getMessage());
			throw new HeroException("Error in propagatesParentProperties");}
	}

	/**
	 * Propagate subProcess properties to subProcess activity.
	 *
	 * @param node the name of the node
	 * @ejb:interface-method view-type="local"
	 * @ejb:transaction type="Required"
	 * @throws HeroException
	 *
	 **/

	public void propagatesSubProcessProperties(String node) throws HeroException {
		log.info("parameter: nodeName=" + node + " start by " + mContext.getCallerPrincipal().getName());
		BnProjectLocal pl = mProject;
		try{
	      this.initProject(node);
		  Collection props = this.getProperties();
		  mProject = pl;   
          Iterator i = props.iterator();
		  while (i.hasNext())
		  {
		    BnProjectPropertyValue prop = (BnProjectPropertyValue)i.next();
		    this.setNodeProperty(node,prop.getTheKey(),prop.getTheValue(),true);
		  }								
		}catch (Exception ans){
			mProject=pl;
			log.error("Error in propagatesSubProcessProperties " + ans.getMessage());
			throw new HeroException("Error in propagatesSubProcessProperties ");}
	}
	
	/**
	 * Set the edge state
	 *
	 * @param bean of the edge
	 * @param new state of the edge
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 * @throws HeroException
	 *
	 **/
	public void setEdgeState(BnEdgeLocal edge, int state)
		throws HeroException {
		log.info("parameters: name=" + edge.getName() + " state=" + state + " start by " + mContext.getCallerPrincipal().getName());
		edge.setState(state);
		jms.sendEdgeEvent(SETEDGESTATE,this.getName(),edge.getName(),edge.getInBnNode().getName(),edge.getOutBnNode().getName(),mContext.getCallerPrincipal().getName());
		log.info("End");
	}

	/**
	 * Set the agent state
	 *
	 * @param agent bean of the Agent
	 * @param new state of the Agent
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 * @throws HeroException
	 *
	 **/
	public void setAgentState(BnAgentLocal agent, int state)
		throws HeroException {
		log.info("parameters: name=" + agent.getName() + " state=" + state + " state=" + state + " start by " + mContext.getCallerPrincipal().getName());
		agent.setState(state);
		//	jms.sendEdgeEvent(SETEDGESTATE,this.getName(),edge.getName(),edge.getInBnNode().getName(),edge.getOutBnNode().getName(),mContext.getCallerPrincipal().getName());
	log.info("End");
	}

	/**
	 * Set the agentEdge state
	 *
	 * @param agentEdge bean of the BnAgentEdge
	 * @param new state of the BnAgentEdge
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 * @throws HeroException
	 *
	 **/
	public void setAgentEdgeState(BnAgentEdgeLocal agentEdge, int state)
		throws HeroException {
		log.info("parameters: name="+ agentEdge.getName()+ " state="+ state + " state=" + state + " start by " + mContext.getCallerPrincipal().getName());
		agentEdge.setState(state);
		//	jms.sendEdgeEvent(SETEDGESTATE,this.getName(),edge.getName(),edge.getInBnNode().getName(),edge.getOutBnNode().getName(),mContext.getCallerPrincipal().getName());
		log.info("End");
	}
	
	/**
	* Set edit node changes. Only for GraphEditor client
	*
	* @param name of the node
	* @param role of the node
	* @param description of the node
	* @param deadline of the node
	* @ejb:interface-method view-type="both"
	* @ejb:transaction type="Required"
	*
	**/
	public void setEditNode(String node, String role, String description, long deadline) throws HeroException {
		log.debug("setEditNode:  node=" + node + " role=" + role +" description=" + description + " deadline=" + deadline);
		this.setNodeDeadline(node,deadline);
		this.setNodeDescription(node,description);
		this.setNodeRole(node,role);		
	}

	/**
	 * Set the edge condition. This condition is evaluated at run-time in order to perform activity transition.
	 * To set the condition you have to use beanshell language (Lightweight Scripting for Java).<br>
	 * <strong>The next line illustrates the use of setEdgeCondition method:<br><br></strong>
	 * 
	 * ...<br>
	 * String condition = "partial_status.equals(\"ok\")";<br>  
	 * <strong>yourProjectSessionInstance</strong>.setEdgeCondition("edgeName",condition);<br><br>
	 * 
	 * The engine evaluates on the fly the condition by using the If statement. In the previous sample,
	 * if the value of the property "partial_status" is equals to "ok" the result of this condition is true.
	 *
	 * @param edge edge name
	 * @param condition condition of the edge
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 * @throws HeroException
	 *
	 **/
	public void setEdgeCondition(String edge, String condition)
		throws HeroException {
		log.info("parameters: name=" + edge + " condition=" + condition + " start by " + mContext.getCallerPrincipal().getName());
		BnEdgeLocal e = mProject.getBnEdge(edge);
		e.setCondition(condition);
		log.info("End");
	}
	
	/**
	 * Set the node deadline. Activity deadline is the lastest date in which the activity must be finished.
	 *
	 * @param name the name of the node
	 * @param date date of the deadline
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 * @throws HeroException
	 *
	 **/
	public void setNodeDeadline(String name, long date) throws HeroException {
		log.info("parameters: name=" + name + " deadline=" + date + " start by " + mContext.getCallerPrincipal().getName());

		BnNodeLocal nd = mProject.getBnNode(name);
		nd.setDeadline(new java.sql.Date(date));
		String projectName = this.getName();
		jms.sendNodeEvent(SETNODEDEADLINE,projectName,name,nd.getType(),nd.getState(),mContext.getCallerPrincipal().getName());

		try {
			String type = name + ":" + projectName;
			removeOldNotification(type);

			/*				ObjectName timer = new ObjectName("jboss:service=timer");
							ObjectName receiver = new ObjectName("jboss:service=TimerReceiver");*/
			ObjectName timer = new ObjectName("jonas:type=service,name=Timer");
			ObjectName receiver =
				new ObjectName("jonas:type=service,name=TimerReceiver");
			List list = MBeanServerFactory.findMBeanServer(null);
			MBeanServer server = (MBeanServer) list.iterator().next();
			Date lNext = new Date(date);
			NotificationFilter filter = new TimerFilter();
			server.addNotificationListener(timer, receiver, filter, null);
			Integer id =(Integer) server.invoke(timer, "addNotification", new Object[] { type,
										//type
										name, //message
										"bonita", //user object
										lNext, //delay time
										},
										new String[] {
										"".getClass().getName(),
										"".getClass().getName(),
										"java.lang.Object",
										Date.class.getName()});
		} catch (Exception e) {
			log.error(e.getMessage());
			e.printStackTrace();
			throw new HeroException(e.getMessage());
		}
		log.info("End");
	}

	/**
	 * Set the node description. Node description represents, explicity, execution related information 
	 * of this task. 
	 *
	 * @param name the name of the node
	 * @param descriptio description of the node
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 * @throws HeroException
	 *
	 **/
	public void setNodeDescription(String name, String description)
		throws HeroException {
		log.info("parameters: name="+ name+ " description="+ description + " start by " + mContext.getCallerPrincipal().getName());
		BnNodeLocal nd = mProject.getBnNode(name);
		nd.setDescription(description);
		log.info("End");
	}

	/**
	 * Set the node hook value.
	 *
	 * @param name name of the node
	 * @param hook hookName of the node
	 * @param value new script value of this hook
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 * @throws HeroException
	 *
	 **/
	public void setNodeInterHookValue(String node, String hook, String value)
		throws HeroException {
		log.info("parameters: node="+ node+ " hook="+ hook+ " value="+ value+ " start by " + mContext.getCallerPrincipal().getName());
		BnNodeLocal nd = mProject.getBnNode(node);
		Collection hooks = nd.getBnInterHooks();
		for (Iterator ih = hooks.iterator(); ih.hasNext();) {
			BnNodeInterHookLocal hk = (BnNodeInterHookLocal) ih.next();
			if (hk.getName().equals(hook)) {
				hk.setScript(value);
			}
		}
		log.info("End");
	}

	/**
	 * Set the hook value. This value is the new hook script associated to all project nodes 
	 *
	 * @param hook hookName of the node
	 * @param value new script value of this hook
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 * @throws HeroException
	 *
	 **/
	public void setInterHookValue(String hook, String value)
		throws HeroException {
		log.info("parameters:  hook=" + hook + " value=" + value+ " start by " + mContext.getCallerPrincipal().getName());
		Collection hooks = mProject.getBnInterHooks();
		for (Iterator ih = hooks.iterator(); ih.hasNext();) {
			BnProjectInterHookLocal hk = (BnProjectInterHookLocal) ih.next();
			if (hk.getName().equals(hook)) {
				hk.setScript(value);
			}
		}
		log.info("End");
	}

	/**
	 * Unset a user role in this project.
	 *
	 * @param userName name of the user
	 * @param roleName name of the role
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 * @throws HeroException
	 *
	 **/

	public void unsetUserRole(String userName, String roleName) throws HeroException{
		log.info("parameters: userName=" + userName + " roleName=" + roleName + " start by " + mContext.getCallerPrincipal().getName());
		BnUserLocal mUser;
		BnUserLocalHome uHome;
		BnRoleLocalHome rHome;
		BnRoleLocal mRole;
		try {
			uHome = hero.interfaces.BnUserUtil.getLocalHome();
			rHome = hero.interfaces.BnRoleUtil.getLocalHome();
		} catch (javax.naming.NamingException be) {
			log.error(be.getMessage());
		  throw new HeroException(be.getMessage());
		}
		try {
			mUser = uHome.findByName(userName);
		} catch (javax.ejb.FinderException fe) {
			log.error("User does not exist " + fe.getMessage());
			throw new HeroException("User does not exist");
		}
		
		Collection userRoles = this.getUserRolesInProject(userName);
		Iterator i = userRoles.iterator();
		while (i.hasNext())
		{
			BnRoleValue role = (BnRoleValue)i.next();
			if (role.getName().equals(roleName))
			{
				try {
					mRole = rHome.findByName(role.getName(),mProject.getId());
					mUser.getBnRoles().remove(mRole);
				} catch (javax.ejb.FinderException fe) {
					log.error("Role does not exist " + fe.getMessage());
					throw new HeroException("Role does not exist");
				}				
			}
		}
		
		jms.sendUserRoleEvent(UNSETUSERROLE,this.getName(),userName,roleName);
		log.info("End");
	}

	/**
	 * Set the node in traditional mode. When a node is traditional the anticipable attribute is false. This
	 * method must be used if you want to execute this activity in a traditional model.
	 *
	 * @param name name of the node
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 * @throws HeroException
	 *
	 **/
	public void setNodeTraditional(String name) throws HeroException {
		log.info("parameter: name=" + name + " start by " + mContext.getCallerPrincipal().getName());
		BnNodeLocal nd = mProject.getBnNode(name);
		nd.setAnticipable(false);
		nd.setTransition(NodeState.make(nd.getType(), false));
		log.info("End");
	}

	/**
	 * Set the node in anticipable mode.
	 *
	 * @param name the name of the node
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 * @throws HeroException
	 *
	 **/
	public void setNodeAnticipable(String name) throws HeroException {
		log.info("parameter: name=" + name + " start by " + mContext.getCallerPrincipal().getName());
		BnNodeLocal nd = mProject.getBnNode(name);
		nd.setAnticipable(true);
		nd.setTransition(NodeState.make(nd.getType(), true));
		log.info("End");
	}

	/**
	 * Set the node in automatic  mode. The responsible of the activity execution is now the engine.
	 *
	 * @param name the name of the node
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 * @throws HeroException
	 *
	 **/
	public void setNodeAutomatic(String name) throws HeroException {
		log.info("parameter: name=" + name + " start by " + mContext.getCallerPrincipal().getName());
		BnNodeLocal nd = mProject.getBnNode(name);
		//nd.setAutomatic(true);
		nd.setTransition(NodeState.make(5, true));
		//nd.setTransition(NodeState.make(nd.getType(),true));
		//	this.addNodeHok(name,"hero.hook.AutomaticHook",hero.interfaces.Constants.Nd.CREATE,Hook.JAVA);
		log.info("End");
	}

	/**
	 * Set the name of the node executor. Set the name of the user which is executing the activity.
	 *
	 * @param name the name of the node
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 * @throws HeroException
	 *
	 **/
	public void setNodeExecutor(String name) throws HeroException {
		log.info("parameter: name=" + name + " start by " + mContext.getCallerPrincipal().getName());
		BnUserLocalHome uHome;
		BnUserLocal user;
		try {
			uHome = BnUserUtil.getLocalHome();
		} catch (javax.naming.NamingException ne) {
			log.error(ne.getMessage());
			throw new HeroException(ne.getMessage());
		}
		BnNodeLocal nd = mProject.getBnNode(name);
		String executor = mContext.getCallerPrincipal().getName();
		try {
			user = uHome.findByName(executor);
		} catch (FinderException fe) {
			log.error("BnUser " + executor + " does not exists " + fe.getMessage());
			throw new HeroException("BnUser " + executor + " does not exists");
		}
		nd.setExecutor(user);
		jms.sendNodeEvent(SETNODEEXECUTOR,this.getName(),name,nd.getType(),nd.getState(),mContext.getCallerPrincipal().getName());
		log.info("End");
	}

	/**
	 * Set the node type. Change the current type of the node (if node is not executing).<br><br>
	 * 
	 * Possibles node types are:<br>
	 * hero.interfaces.Constants.Nd.AND_JOIN_NODE<br>
	 * hero.interfaces.Constants.Nd.OR_JOIN_NODE<br>
	 * hero.interfaces.Constants.Nd.AND_JOIN_AUTOMATIC_NODE<br>
	 * hero.interfaces.Constants.Nd.OR_JOIN_AUTOMATIC_NODE<br>
	 *
	 * @param name the name of the node
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 * @throws HeroException
	 *
	 **/
	public void setNodeType(String name, int type) throws HeroException {
		log.info("parameters: name=" + name + " type" + type + " start by " + mContext.getCallerPrincipal().getName());
		BnNodeLocal nd = mProject.getBnNode(name);
		nd.setType(type);
		nd.setTransition(NodeState.make(nd.getType(), true));
		nd.setActivation(EdgeState.make(nd.getType()));
		log.info("End");
	}

	/**
	 * Set a property of a node. A property is a pair key/value representing workflow relevant data. By using
	 * propagate argument we can specify if we want to propagate this property.
	 *
	 * @param nodeName the name of the node
	 * @param key the key of the property
	 * @param value the name of the property
	 * @param propagate if it is a propagation property
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 * @throws HeroException
	 *
	 **/
	public void setNodeProperty(String nodeName,String key,String value,boolean propagate)
		throws HeroException {

		BnNodePropertyLocalHome pHome;
		BnNodePropertyLocal propertyLocal;

		log.info("parameters: node="+ nodeName+ " key="+ key+ " value="+ value + " start by " + mContext.getCallerPrincipal().getName());

		BnNodeLocal nd = mProject.getBnNode(nodeName);

		try {
			pHome = BnNodePropertyUtil.getLocalHome();
		} catch (javax.naming.NamingException ne) {
			log.error(ne.getMessage());
			throw new HeroException(ne.getMessage());
		}
		Collection c = null;
		try {
			c = pHome.findByTheKey(nd.getId(), key);
		} catch (FinderException fe) {
			log.error(fe.getMessage());
			throw new HeroException(fe.getMessage());
		}
		if (!c.isEmpty()) {
			propertyLocal = (BnNodePropertyLocal) (c.toArray())[0];
			propertyLocal.setTheValue(value);
			propertyLocal.setPropagate(propagate);
		} else {
			try {
				propertyLocal = pHome.create(key, value);
				propertyLocal.setBnNode(nd);
				propertyLocal.setPropagate(propagate);
			} catch (InvalidValueException ie) {
				log.error(ie.getMessage());
				throw new HeroException(ie.getMessage());
			} catch (javax.ejb.CreateException ce) {
				log.error(ce.getMessage());
				throw new HeroException(ce.getMessage());
			}
		}
		log.info("End");
	}

	/**
	 * Set a property of a node. A property is a pair key/value representing workflow relevant data. With
	 * this method the property is propagated within others nodes.
	 *
	 * @param nodeName the name of the node
	 * @param key the key of the property
	 * @param value the name of the property
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 * @throws HeroException
	 *
	 **/

	public void setNodeProperty(String nodeName, String key, String value)
		throws HeroException {
		BnNodePropertyLocalHome pHome;
		BnNodePropertyLocal propertyLocal = null;

		log.info("parameters: node="+ nodeName+ " key="+ key+ " value="+ value + " start by " + mContext.getCallerPrincipal().getName());
		BnNodeLocal nd = mProject.getBnNode(nodeName);

		try {
			pHome = BnNodePropertyUtil.getLocalHome();
		} catch (javax.naming.NamingException ne) {
			log.error(ne.getMessage());
			throw new HeroException(ne.getMessage());
		}
		Collection c = null;
		try {
			c = pHome.findByTheKey(nd.getId(), key);
		} catch (FinderException fe) {
			log.error(fe.getMessage());
			throw new HeroException(fe.getMessage());
		}

		if (!c.isEmpty()) {

			propertyLocal = (BnNodePropertyLocal) (c.toArray())[0];
			propertyLocal.setTheValue(value);

		} else {
			try {
				propertyLocal = pHome.create(key, value);
				propertyLocal.setBnNode(nd);

			} catch (InvalidValueException ie) {
				log.error(ie.getMessage());
				throw new HeroException(ie.getMessage());
			} catch (javax.ejb.CreateException ce) {
				log.error(ce.getMessage());
				throw new HeroException(ce.getMessage());
			}
		}
		log.info("End");
	}

	/**
	 * Set a property of the project. A property is a pair key/value representing workflow relevant data. This
	 * methods creates a new property that can be accessed from all activites of the project.
	 *
	 * @param key the key of the property
	 * @param value the name of the property
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 * @throws HeroException
	 *
	 **/
	public void setProperty(String key, String value) throws HeroException {
		log.info("parameters: key=" + key + " value=" + value + " start by " + mContext.getCallerPrincipal().getName());

		BnProjectPropertyLocalHome pHome;
		BnProjectPropertyLocal ppLocal = null;

		try {
			pHome = BnProjectPropertyUtil.getLocalHome();
		} catch (javax.naming.NamingException ne) {
			log.error(ne.getMessage());
			throw new HeroException(ne.getMessage());
		}
		Collection c = null;
		try {
			c = pHome.findByTheKey(mProject.getId(), key);
		} catch (FinderException fe) {
			log.error(fe.getMessage());
			throw new HeroException(fe.getMessage());
		}

		if (!c.isEmpty()) {

			ppLocal = (BnProjectPropertyLocal) (c.toArray())[0];
			ppLocal.setTheValue(value);

		} else {
			try {
				ppLocal = pHome.create(key, value);
				ppLocal.setBnProject(mProject);

			} catch (InvalidValueException ie) {
				log.error(ie.getMessage());
				throw new HeroException(ie.getMessage());
			} catch (javax.ejb.CreateException ce) {
				log.error(ce.getMessage());
				throw new HeroException(ce.getMessage());
			}
		}
		log.info("End");
	}

	/**
	 * Execute a node hook.
	 *
	 * @param nodeName the name of the node
	 * @param event the event 
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 * @throws HeroException
	 *
	 **/
	public void hookEvent(String nodeName, String event) throws HeroException {
		log.info("parameters: node=" + nodeName + " event=" + event + " start by " + mContext.getCallerPrincipal().getName());
		Collection hooks;
		BnNodeLocal node;

		node = mProject.getBnNode(nodeName);
		hooks = node.getBnHooks();
		for (Iterator i = hooks.iterator(); i.hasNext();) {
			BnNodeHookLocal hk = (BnNodeHookLocal) i.next();
			if (hk.getEvent().equalsIgnoreCase(event)) {
				hero.hook.Hook h =
					hero.hook.Hook.make(hk.getName(), event, hk.getType());
				h.execute(this, event, node);
			}
		}
		log.info("End");
	}

	// -------------------------------------------------------------------------
	// Get business methods
	// -------------------------------------------------------------------------

	/**
	 * Get the node description. Node description represents, explicity, execution related information 
	 * of this task. 
	 *
	 * @param name of the node
	 * @return String - description of the node
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 * @throws HeroException
	 *
	 **/
	public String getNodeDescription(String name) throws HeroException {
		log.info("parameter: name=" + name + " start by " + mContext.getCallerPrincipal().getName());
		BnNodeLocal nd = mProject.getBnNode(name);
		log.info("End");
		return (nd.getDescription());
	}

	/**
	 * Get the node hook value. This method returns the hook script associated to this node
	 *
	 * @param name the name of the node
	 * @param hook hookName of the node
	 * @return String - script associated to this hook
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 * @throws HeroException
	 *
	 **/
	public String getNodeInterHookValue(String node, String hook)
		throws HeroException {
		String value = null;
		log.info("parameters: node=" + node + " hook=" + hook + " start by " + mContext.getCallerPrincipal().getName());
		BnNodeLocal nd = mProject.getBnNode(node);
		Collection hooks = nd.getBnInterHooks();
		for (Iterator ih = hooks.iterator(); ih.hasNext();) {
			BnNodeInterHookLocal hk = (BnNodeInterHookLocal) ih.next();
			if (hk.getName().equals(hook)) {
				value = hk.getScript();
			}
		}
		log.info("End");
		return value;
	}

	/**
	  * Get the inter hook value script. This method returns the hook script associated to all project nodes.
	  *
	  * @param hook hook name
	  * @return String - script associated to this hook
	  * @ejb:interface-method view-type="both"
	  * @ejb:transaction type="Supports"
	  * @throws HeroException
	  *
	  **/
	public String getInterHookValue(String hook) throws HeroException {
		String value = null;
		log.info("parameter:  hook=" + hook + " start by " + mContext.getCallerPrincipal().getName());
		Collection hooks = mProject.getBnInterHooks();
		for (Iterator ih = hooks.iterator(); ih.hasNext();) {
			BnProjectInterHookLocal hk = (BnProjectInterHookLocal) ih.next();
			if (hk.getName().equals(hook)) {
				value = hk.getScript();
			}
		}
		log.info("End");
		return value;
	}


	/**
	 * Get the ProjectValue (historique). Returns project information: project attributes, nodes, edges,
	 * hooks, properties...
	 *
	 * @return BnProjectValue object - project data
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 * @throws HeroException
	 *
	 **/
	public BnProjectValue getDetails() throws HeroException {
		log.info(" start by " + mContext.getCallerPrincipal().getName());
		log.info("End");
		return mProject.getBnProjectValue();
	}

	/**
	 * Get ProjectName
	 * @return String - project name
	 *
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 * @throws HeroException
	 *
	 **/
	public String getName() throws HeroException {
		log.info(" start by " + mContext.getCallerPrincipal().getName());
		return mProject.getName();
	}

	/**
	 * Get creator of the project. The user that created the project
	 * @return String - project creator
	 *
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 * @throws HeroException
	 *
	 **/
	
	public String getCreator() throws HeroException {
		log.info(" start by " + mContext.getCallerPrincipal().getName());
		log.info("End");
		return mProject.getCreator();
	}

	/**
	 * Get all roles of the project
	 * @return Collection of BnRoleLocal objects - roles of the project
	 *
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 * @throws HeroException
	 *
	**/
	public Collection getRoles() throws HeroException {
		log.info(" start by " + mContext.getCallerPrincipal().getName());
		Collection roles = mProject.getBnRoles();
		ArrayList result = new ArrayList();
		for (Iterator i = roles.iterator(); i.hasNext();) {
			result.add((BnRoleLocal) i.next());
		}
		log.info("End");
		return result;
	}

	/**
	 * Get all roles names of the project
	 * @return String Collection - roles names of this projects
	 *
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 * @throws HeroException
	 *
	 **/
	public Collection getRolesNames() throws HeroException {
		log.info(" start by " + mContext.getCallerPrincipal().getName());
		Collection roles = mProject.getBnRoles();
		ArrayList result = new ArrayList();
		for (Iterator i = roles.iterator(); i.hasNext();) {
			result.add(((BnRoleLocal) i.next()).getName());
		}
		log.info("End");
		return result;
	}


	/**
	 * Get all user roles. This method returns all the roles for this user in differents projects.
	 *
	 * @param userName The name of the user
	 * @return Collection of BnRoleLOcal objects - roles of the user in differents projects
	 *
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 * @throws HeroException
	 *
	 **/
	public Collection getUserRoles(String userName) throws HeroException {
		log.info(" start by " + mContext.getCallerPrincipal().getName());
		BnUserLocalHome uHome;
		BnUserLocal user;
		try {
			uHome = (BnUserLocalHome) hero.interfaces.BnUserUtil.getLocalHome();
		} catch (javax.naming.NamingException be) {
			log.error(be.getMessage());
			throw new HeroException(be.getMessage());
		}
		try {
			user = uHome.findByName(userName);
		} catch (FinderException f) {
			log.error("FinderException " + f.getMessage());
			throw new HeroException("FinderException");
		}
		Collection roles = user.getBnRoles();
		ArrayList result = new ArrayList();
		for (Iterator i = roles.iterator(); i.hasNext();) {
			result.add((BnRoleLocal) i.next());
		}
		log.info("End");
		return result;
	}

	/**
	 * Get user roles in project. Returns the roles of this user in the current project.
	 *
	 * @param userName The name of the user
	 * @return Collection of BnRoleValue objects - roles of the user in this project
	 *
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 * @throws HeroException
	 *
	 **/
	public Collection getUserRolesInProject(String userName)
		throws HeroException {
		log.info(" start by " + mContext.getCallerPrincipal().getName());
		BnUserLocalHome uHome;
		BnUserLocal user;
		ArrayList result = new ArrayList();
		try {
			uHome = (BnUserLocalHome) hero.interfaces.BnUserUtil.getLocalHome();
		} catch (javax.naming.NamingException be) {
			log.error(be.getMessage());
			throw new HeroException(be.getMessage());
		}
		try {
			user = uHome.findByName(userName);
		} catch (FinderException f) {
			log.error("FinderException " + f.getMessage());
			throw new HeroException("FinderException");
		}
		Collection userRoles = user.getBnRoles();
		Collection projectRoles = mProject.getBnRoles();
		for (Iterator i = userRoles.iterator(); i.hasNext();) {
			BnRoleLocal ur = (BnRoleLocal) i.next();
			for (Iterator j = projectRoles.iterator(); j.hasNext();) {
				BnRoleLocal up = (BnRoleLocal) j.next();
				if (ur.getId() == up.getId())
					result.add(ur.getBnRoleValue());
			}
		}
		log.info("End");
		return result;
	}

	/**
	 * Get user roles in project names. Returns the role names of the user in the current project
	 *
	 * @param userName The name of the user
	 * @return String Collection - role names of this user in the current project
	 *
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 * @throws HeroException
	 *
	 **/
	public Collection getUserRolesInProjectNames(String userName)
		throws HeroException {
		log.info(" start by " + mContext.getCallerPrincipal().getName());
		BnUserLocalHome uHome;
		BnUserLocal user;
		ArrayList result = new ArrayList();
		try {
			uHome = (BnUserLocalHome) hero.interfaces.BnUserUtil.getLocalHome();
		} catch (javax.naming.NamingException be) {
			log.error(be.getMessage());
			throw new HeroException(be.getMessage());
		}
		try {
			user = uHome.findByName(userName);
		} catch (FinderException f) {
			log.error("FinderException " + f.getMessage());
			throw new HeroException("FinderException");
		}

		Collection userRoles = user.getBnRoles();
		Collection projectRoles = mProject.getBnRoles();
		for (Iterator i = userRoles.iterator(); i.hasNext();) {
			BnRoleLocal ur = (BnRoleLocal) i.next();
			for (Iterator j = projectRoles.iterator(); j.hasNext();) {
				BnRoleLocal up = (BnRoleLocal) j.next();
				if (ur.getId() == up.getId())
					result.add(ur.getBnRoleValue().getName());
			}
		}
		log.info("End");
		return result;
	}

	/**
	 * Get node role. Returns the role information.
	 *
	 * @param nodeName The name of the node
	 * @return BnRoleLocal object - the role data
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 * @throws HeroException
	 *
	 **/
	public BnRoleLocal getNodeRole(String nodeName) throws HeroException {
		log.info("parameter: nodeName=" + nodeName+ " start by " + mContext.getCallerPrincipal().getName());
		BnNodeLocal nd = mProject.getBnNode(nodeName);
		log.info("End");
		return ((BnRoleLocal) nd.getBnRole());
	}
	
	/**
	 * Get the edge Condition. This condition is evaluated at run-time in order to perform activity transition.
	 * This condition follows the beanshell syntax (Lightweight Scripting for Java).<br>
	 *
	 * @param edge the edge/transition name
	 * @return String - edge condition
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 * @throws HeroException
	 *
	 **/
	public String getEdgeCondition(String edge)
		throws HeroException {
		log.info("parameter: name=" + edge+ " start by " + mContext.getCallerPrincipal().getName());
		BnEdgeLocal e = mProject.getBnEdge(edge);
		log.info("End");
		return(e.getCondition());
	}

	/**
	 * Returns if the node is set to be executed in anticipated mode.
	 *
	 * @param name name of the node
	 * @return boolean - if anticipable node is set to this node.
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 * @throws HeroException
	 *
	 **/
	public boolean getNodeAnticipable(String name) throws HeroException {
		log.info("parameter: name=" + name+ " start by " + mContext.getCallerPrincipal().getName());
		BnNodeLocal nd = mProject.getBnNode(name);
		log.info("End");
		return (nd.getAnticipable());
	}

	/**
	 * Get node role name. Obtains the role name of this node.
	 *
	 * @param nodeName The name of the node
	 * @return String - the role name of this node
	 *
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 * @throws HeroException
	 *
	 **/
	public String getNodeRoleName(String nodeName) throws HeroException {
		log.info("parameter: nodeName=" + nodeName+ " start by " + mContext.getCallerPrincipal().getName());
		BnNodeLocal nd = mProject.getBnNode(nodeName);
		log.info("End");
		return (((BnRoleLocal) nd.getBnRole()).getName());
	}

	/**
	 * Get node deadline. Activity deadline is the lastest date in which the activity must be finished.
	 *
	 * @param nodeName The name of the node
	 * @return String - deadline of this node
	 *
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 * @throws HeroException
	 *
	 **/
	public String getNodeDeadline(String nodeName) throws HeroException {
		log.info("parameter: nodeName=" + nodeName+ " start by " + mContext.getCallerPrincipal().getName());
		BnNodeLocal nd = mProject.getBnNode(nodeName);
		log.info("End");
		if (nd.getDeadline() != null)
			return (nd.getDeadline().toString());
		else
			return (null);
	}

	/**
	 * Get the node Value. Returns node information.
	 *
	 * @param name the name of the Node
	 * @return BnNodeValue object - node data.
	 *
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 * @throws HeroException
	 *
	 **/
	public BnNodeValue getNodeValue(String name) throws HeroException {
		log.info("parameter: name=" + name + " start by " + mContext.getCallerPrincipal().getName());
		BnNodeLocalHome nHome;
		BnNodeLocal mNode;
		BnNodeValue nv;
		try {
			nHome = hero.interfaces.BnNodeUtil.getLocalHome();
		} catch (javax.naming.NamingException be) {
			log.error(be.getMessage());
			throw new HeroException(be.getMessage());
		}
		try {
			mNode = nHome.findByName(name, mProject.getId());
		} catch (FinderException fe) {
			log.error("In Node does not exist " + fe.getMessage());
			throw new HeroException("In Node does not exist");
		}
		nv = mNode.getBnNodeValue();
		log.info("End");
		return nv;
	}

	/**
	 * Get the Agent Value.
	 *
	 * @param name the name of the Agent
	 * @return BnAgentValue object - agent data.
	 *
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 * @throws HeroException
	 *
	 **/
	public BnAgentValue getAgentValue(String name) throws HeroException {
		log.info("parameter: name=" + name + " start by " + mContext.getCallerPrincipal().getName());
		BnAgentLocalHome nHome;
		BnAgentLocal mAgent;
		BnAgentValue av;
		try {
			nHome = hero.interfaces.BnAgentUtil.getLocalHome();
		} catch (javax.naming.NamingException be) {
			log.error(be.getMessage());
			throw new HeroException(be.getMessage());
		}
		try {
			mAgent = nHome.findByName(name, mProject.getId());
		} catch (FinderException fe) {
			log.error("In BnAgent does not exist " + fe.getMessage());
			throw new HeroException("In BnAgent does not exist");
		}
		av = mAgent.getBnAgentValue();
		log.info("End");
		return av;
	}

	/**
	 * Get the AgentEdge Value
	 *
	 * @param name the name of the BnAgentEdge
	 * @return BnAgentEdgeValue object - agentEdge data.
	 *
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 * @throws HeroException
	 *
	 **/
	public BnAgentEdgeValue getAgentEdgeValue(String name)
		throws HeroException {
		log.info("parameter: name=" + name);
		BnAgentEdgeLocalHome nHome;
		BnAgentEdgeLocal mAgentEdge;
		BnAgentEdgeValue av;
		try {
			nHome = hero.interfaces.BnAgentEdgeUtil.getLocalHome();
		} catch (javax.naming.NamingException be) {
			log.error(be.getMessage());
			throw new HeroException(be.getMessage());
		}
		try {
			mAgentEdge = nHome.findByName(name, mProject.getId());
		} catch (FinderException fe) {
			log.error("In BnAgentEdge does not exist " + fe.getMessage());
			throw new HeroException("In BnAgentEdge does not exist");
		}
		av = mAgentEdge.getBnAgentEdgeValue();
		log.info("End");
		return av;
	}

	/**
	 * Get the node Light Value. Returns node basic information.
	 *
	 * @param name the name of the Node
	 * @return BnNodeLightValue object - agentEdge data.
	 *
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 * @throws HeroException
	 *
	 **/
	public BnNodeLightValue getNodeLightValue(String name)
		throws HeroException {
		log.info("parameter: name=" + name);
		BnNodeLocalHome nHome;
		BnNodeLocal mNode;
		BnNodeLightValue nv;
		try {
			nHome = hero.interfaces.BnNodeUtil.getLocalHome();
		} catch (javax.naming.NamingException be) {
			log.error(be.getMessage());
			throw new HeroException(be.getMessage());
		}
		try {
			mNode = nHome.findByName(name, mProject.getId());
		} catch (FinderException fe) {
			log.error("In Node does not exist " + fe.getMessage());
			throw new HeroException("In Node does not exist");
		}
		nv = mNode.getBnNodeLightValue();
		log.info("End");
		return nv;
	}

	
	/**
	 * Get the edge value.
	 *
	 * @param name the name of the edge
	 * @return BnEdgeValue object - edge data.
	 *
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 * @throws HeroException
	 *
	 **/
	public BnEdgeValue getEdgeValue(String name) throws HeroException {
		log.info("parameter: name=" + name + " start by " + mContext.getCallerPrincipal().getName());
		BnEdgeLocalHome eHome;
		BnEdgeLocal mEdge;
		BnEdgeValue ev;
		try {
			eHome = hero.interfaces.BnEdgeUtil.getLocalHome();
		} catch (javax.naming.NamingException be) {
			log.error(be.getMessage());
			throw new HeroException(be.getMessage());
		}
		try {
			mEdge = eHome.findByName(name, mProject.getId());
		} catch (FinderException fe) {
			log.error("BnEdge does not exist " + fe.getMessage());
			throw new HeroException("BnEdge does not exist");
		}
		ev = mEdge.getBnEdgeValue();
		log.info("End");
		return ev;
	}

	/**
	 * Get the node executor. Get the name of the user which is executing the activity.
	 *
	 * @param name the name of the node
	 * @return String - the user executor of this node
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 * @throws HeroException
	 *
	 **/
	public String getNodeExecutor(String name) throws HeroException {
		log.info("parameter: name=" + name + " start by " + mContext.getCallerPrincipal().getName());
		BnNodeLocal nd = mProject.getBnNode(name);
		BnUserLocal ul = nd.getExecutor();
		log.info("End");
		return (ul.getName());
	}

	/**
	 * Get the state of the node. The possible states for a node are:<br><br>
	 *
	 * hero.interfaces.Constants.Nd.INITIAL<br>
	 * hero.interfaces.Constants.Nd.READY<br>
	 * hero.interfaces.Constants.Nd.DEAD<br>
	 * hero.interfaces.Constants.Nd.ANTICIPABLE<br>
	 * hero.interfaces.Constants.Nd.ANTICIPATING<br>
	 * hero.interfaces.Constants.Nd.EXECUTING<br>
	 * hero.interfaces.Constants.Nd.TERMINATED<br>
	 *
	 * @param name of the node
	 * @return int - The state of the node
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 * @throws HeroException
	 *
	 **/
	public int getNodeState(String name) throws HeroException {
		log.info("parameter: name=" + name + " start by " + mContext.getCallerPrincipal().getName());
		BnNodeLocal nd = mProject.getBnNode(name);
		log.info("End");
		return (nd.getState());
	}

	/**
	 * Get the type of the node. Possibles node types are:<br><br>
	 * 
	 * hero.interfaces.Constants.Nd.AND_JOIN_NODE<br>
	 * hero.interfaces.Constants.Nd.OR_JOIN_NODE<br>
	 * hero.interfaces.Constants.Nd.AND_JOIN_AUTOMATIC_NODE<br>
	 * hero.interfaces.Constants.Nd.OR_JOIN_AUTOMATIC_NODE<br>
	 *
	 * @param name of the node
	 * @return int - The type of the node
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 * @throws HeroException
	 *
	 **/
	public int getNodeType(String name) throws HeroException {
		log.info("parameter: name=" + name + " start by " + mContext.getCallerPrincipal().getName());
		BnNodeLocal nd = mProject.getBnNode(name);
		log.info("End");
		return (nd.getType());
	}

	/**
	 * Get all users of the project. Obtains the user names of the project
	 * @return String Collection - user names in project
	 *
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 * @throws HeroException
	 *
	 **/
	public Collection getUsers() throws HeroException {
		log.info(" start by " + mContext.getCallerPrincipal().getName());
		Collection users = mProject.getBnUsers();
		ArrayList result = new ArrayList();
		for (Iterator i = users.iterator(); i.hasNext();) {
			result.add(((BnUserLocal) i.next()).getName());
		}
		log.info("End");
		return result;
	}

	/**
	  * Return all registered users. Get user names of Bonita System
	  * @return String Collection - all user names
	  *
	  * @ejb:interface-method view-type="both"
	  * @ejb:transaction type="Supports"
	  * @throws HeroException
	  *
	  **/
	public Collection getAllUsers() throws HeroException {
		log.info(" start by " + mContext.getCallerPrincipal().getName());
		BnUserLocalHome uHome;
		Collection users;
		try {
			uHome = (BnUserLocalHome) hero.interfaces.BnUserUtil.getLocalHome();
		} catch (javax.naming.NamingException be) {
			log.error(be.getMessage());
			throw new HeroException(be.getMessage());
		}
		try {
			users = uHome.findAll();
		} catch (FinderException f) {
			log.error("FinderException in findByAll users " + f.getMessage());
			throw new HeroException("FinderException in findByAll users");
		}
		ArrayList result = new ArrayList();
		for (Iterator i = users.iterator(); i.hasNext();) {
			result.add(((BnUserLocal) i.next()).getName());
		}
		log.info("End");
		return result;
	}

	/**
	 * Get all node names of the project
	 *
	 * @return String Collection - all node names
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 * @throws HeroException
	 *
	 **/
	public Collection getNodesNames() throws HeroException {
		log.info(" start by " + mContext.getCallerPrincipal().getName());

		//	Vector names = new Vector();
		ArrayList names = new ArrayList();
		Collection nodes = mProject.getBnNodes();
		Iterator i = nodes.iterator();
		while (i.hasNext()) {
			BnNodeLocal n = (BnNodeLocal) i.next();
			names.add(n.getName());
		}
		log.info("End");
		return ((Collection) names);
	}

	/**
	 * Get all edges names of the project
	 * @return String Collection - all edges names of the project 
	 *
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 * @throws HeroException
	 *
	 **/
	public Collection getEdgesNames() throws HeroException {
		log.info(" start by " + mContext.getCallerPrincipal().getName());
		//	Vector names = new Vector();
		ArrayList names = new ArrayList();
		Collection edges = mProject.getBnEdges();
		Iterator i = edges.iterator();
		while (i.hasNext()) {
			BnEdgeLocal n = (BnEdgeLocal) i.next();
			names.add(n.getName());
		}
		log.info("End");
		return ((Collection) names);
	}

	/**
	 * Get edge in node. Returns the name of in node
	 * @param edgeName the name ot the edge
	 * @return String - the name of in node
	 *
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 * @throws HeroException
	 *
	 **/
	public String getEdgeInNode(String edgeName) throws HeroException {
		log.info("parameter: edgeName" + edgeName + " start by " + mContext.getCallerPrincipal().getName());
		BnEdgeLocal e = mProject.getBnEdge(edgeName);
		log.info("End");
		return (((BnNodeLocal) e.getInBnNode()).getName());
	}

	/**
	 * Get edge out node. Returns the name of out node
	 * @param edgeName the name ot the edge
	 * @return String - the name of out node
	 *
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 * @throws HeroException
	 *
	 **/
	public String getEdgeOutNode(String edgeName) throws HeroException {
		log.info("parameter: edgeName" + edgeName + " start by " + mContext.getCallerPrincipal().getName());
		BnEdgeLocal e = mProject.getBnEdge(edgeName);
		log.info("End");
		return (((BnNodeLocal) e.getOutBnNode()).getName());
	}

	/**
	 * Get all in edges of the node
	 *
	 * @param name the name ot the node
	 * @return String Collection - get in edges of the node
	 *
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 * @throws HeroException
	 *
	 **/
	public Collection getNodeInEdges(String name) throws HeroException {
		log.info("parameter: name=" + name + " start by " + mContext.getCallerPrincipal().getName());

		Collection in = new ArrayList();
		BnNodeLocal nd = mProject.getBnNode(name);

		Collection edgesIn = nd.getInBnEdges();
		Iterator i = edgesIn.iterator();
		while (i.hasNext())
			in.add(((BnEdgeLocal) i.next()).getName());
		log.info("End");
		return (in);
	}

	/**
	 * Get all out edges of the node
	 
	 * @param name the name ot the node
	 * @return String Collection - get out edges of the node
	 *
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 * @throws HeroException
	 *
	 **/
	public Collection getNodeOutEdges(String name) throws HeroException {
		log.info("parameter: name=" + name + " start by " + mContext.getCallerPrincipal().getName());

		Collection out = new ArrayList();
		BnNodeLocal nd = mProject.getBnNode(name);

		Collection edgesOut = nd.getOutBnEdges();
		Iterator i = edgesOut.iterator();
		while (i.hasNext())
			out.add(((BnEdgeLocal) i.next()).getName());
		log.info("End");
		return (out);
	}

	/**
	 * Get Node properties of the project. Get a list of pair key/value properties associated to the node.
	 *
	 * @param nodeName the name of the node
	 * @return Collection of BnNodePropertyValue objects - all node properties
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 * @throws HeroException
	 *
	 **/
	public Collection getNodeProperties(String nodeName) throws HeroException {
		   log.info("parameter: name=" + nodeName + " start by " + mContext.getCallerPrincipal().getName());
		Collection prp = new ArrayList();
		BnNodePropertyLocal pl;

		BnNodeLocal nd = mProject.getBnNode(nodeName);

		Collection pls = nd.getBnProperties();

		for (Iterator i = pls.iterator(); i.hasNext();) {
			pl = (BnNodePropertyLocal) i.next();
			prp.add(pl.getBnNodePropertyValue());
		}
		log.info("End");
		return (prp);
	}
	
	/**
	 * Get Node property value of the project. Get a pair key/value properties associated to the node.
	 *
	 * @param nodeName the name of the node
	 * @param key Property key
	 * @return BnNodePropertyValue object - node property data 
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 * @throws HeroException
	 *
	 **/
	public BnNodePropertyValue getNodeProperty(String nodeName, String key) throws HeroException {
		log.info("parameters: node=" + nodeName + " key=" + key + " start by " + mContext.getCallerPrincipal().getName());
		BnNodePropertyLocal pl;
		BnNodeLocal nd = mProject.getBnNode(nodeName);
		Collection pls = nd.getBnProperties();

		for (Iterator i = pls.iterator(); i.hasNext();) {
			pl = (BnNodePropertyLocal) i.next();
			if (pl.getTheKey().equals(key))
			{
			   log.info("End");
			   return (pl.getBnNodePropertyValue());
			}
		}
		log.info("End");
		return (null);
	}
	
	/**
	 * Get property value of the project. Get a pair key/value properties associated to thi project.
	 *
	 * @param nodeName the name of the node
	 * @param key Property key
	 * @return BnNodePropertyValue object - node property data 
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 * @throws HeroException
	 *
	 **/
	public BnProjectPropertyValue getProperty(String key) throws HeroException {
		log.info("parameter: key=" + key + " start by " + mContext.getCallerPrincipal().getName());
		BnProjectPropertyLocal pl;
		Collection pls = mProject.getBnProperties();

		for (Iterator i = pls.iterator(); i.hasNext();) {
			pl = (BnProjectPropertyLocal) i.next();
			if (pl.getTheKey().equals(key))
			{
			   log.info("End");
			   return (pl.getBnProjectPropertyValue());
			}
		}
		log.info("End");
		return (null);
	}
	
	/**
		 * Get Node hooks of the project.  <strong> This functionality is disabled.
		 * Use {@link #getNodeInterHooks(String) getNodeInterHook}
		 * method to get the list of beanshell hooks of the node. </strong>
		 *
		 * @param nodeName the name of the node
		 * @return Collection of BnNodeHookValue - all node hooks
		 * @ejb:interface-method view-type="both"
		 * @ejb:transaction type="Supports"
		 * @throws HeroException
		 *
		 **/
		public Collection getNodeHooks(String nodeName) throws HeroException {
				log.info("parameter: node=" + nodeName  + " start by " + mContext.getCallerPrincipal().getName());
			Collection hk = new ArrayList();
			BnNodeHookLocal pl;

			BnNodeLocal nd = mProject.getBnNode(nodeName);

			Collection pls = nd.getBnHooks();

			for (Iterator i = pls.iterator(); i.hasNext();) {
				pl = (BnNodeHookLocal) i.next();
				hk.add(pl.getBnNodeHookValue());
			}
			log.info("End");
			return (hk);
		}

		/**
		 * Get Interactive Node hooks of the project
		 *
		 * @param nodeName the name of the node
		 * @return Collection of BnNodeInterHookValue objects - all node hooks
		 * @ejb:interface-method view-type="both"
		 * @ejb:transaction type="Supports"
		 * @throws HeroException
		 *
		 **/
		public Collection getNodeInterHooks(String nodeName) throws HeroException {
		log.info("parameter: node=" + nodeName + " start by " + mContext.getCallerPrincipal().getName());
			Collection hk = new ArrayList();
			BnNodeInterHookLocal pl;

			BnNodeLocal nd = mProject.getBnNode(nodeName);

			Collection pls = nd.getBnInterHooks();

			for (Iterator i = pls.iterator(); i.hasNext();) {
				pl = (BnNodeInterHookLocal) i.next();
				hk.add(pl.getBnNodeInterHookValue());
			}
			log.info("End");
			return (hk);
		}
		
	/**
	 * Get role mappers of the project. 
	 *
	 * @return Collection of BnRoleMapperValue - all role mappers
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 * @throws HeroException
	 *
	 **/
	public Collection getRoleMappers() throws HeroException {
		log.info(" start by " + mContext.getCallerPrincipal().getName());
		Collection cm = new ArrayList();
		BnRoleMapperLocal mapperL = null;
		BnRoleLocal roleL = null;	
		Collection cr = mProject.getBnRoles();

		for (Iterator i = cr.iterator(); i.hasNext();) {
			roleL = (BnRoleLocal) i.next();
			// get the mapper if exists
			mapperL = roleL.getBnRoleMapper();
			if ( mapperL != null){
			cm.add(mapperL.getBnRoleMapperValue());	
			}
		}
		log.info("End");
		return (cm);
	}
	
		/**
		 * Get Interactive hooks of the project. Returns hooks assigned to project nodes. 
		 *
		 * @return Collection of ProjectInterHookValue - project inter hooks
		 * @ejb:interface-method view-type="both"
		 * @ejb:transaction type="Supports"
		 * @throws HeroException
		 *
		 **/
		public Collection getInterHooks() throws HeroException {
		log.info(" start by " + mContext.getCallerPrincipal().getName());
			Collection hk = new ArrayList();
			BnProjectInterHookLocal pl;
			Collection pls = mProject.getBnInterHooks();
		
			for (Iterator i = pls.iterator(); i.hasNext();) {
				pl = (BnProjectInterHookLocal) i.next();
				hk.add(pl.getBnProjectInterHookValue());
			}
			log.info("End");
			return (hk);
		}


		/**
		 * Get Node inter hook of the project
		 *
		 * @param nodeName the name of the node
		 * @param interHook name of the hooks
		 * @return BnNodeInterHookValue object - node hook data
		 * @ejb:interface-method view-type="both"
		 * @ejb:transaction type="Supports"
		 * @throws HeroException
		 *
		 **/
		public BnNodeInterHookValue getNodeInterHook(String nodeName, String interHook) throws HeroException {
			log.info("parameters: node=" + nodeName + " key=" + interHook + " start by " + mContext.getCallerPrincipal().getName());
			BnNodeInterHookLocal hl;
			BnNodeLocal nd = mProject.getBnNode(nodeName);
			Collection hls = nd.getBnInterHooks();
	
			for (Iterator i = hls.iterator(); i.hasNext();) {
				hl = (BnNodeInterHookLocal) i.next();
				if (hl.getName().equals(interHook))
				   return (hl.getBnNodeInterHookValue());
			}
			log.info("End");
			return (null);
		}

	/**
	 * Get properties key of the project. A property is a pair key/value representing workflow relevant data. This
	 * methods obtains properties key of the project.
	 * @return Collection of ProjectPropertyValue - all project properties
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 * @throws HeroException
	 *
	 **/
	public Collection getPropertiesKey() throws HeroException {
		log.info("start by " + mContext.getCallerPrincipal().getName());
		Collection prp = new ArrayList();
		BnProjectPropertyLocal ppl;

		Collection pls = mProject.getBnProperties();
		for (Iterator i = pls.iterator(); i.hasNext();) {
			ppl = (BnProjectPropertyLocal) i.next();
			prp.add(ppl.getBnProjectPropertyValue().getTheKey());
		}
		log.info("End");
		return (prp);
	}
	
	/**
	 * Get properties of the project. A property is a pair key/value representing workflow relevant data. This
	 * methods obtains properties of the project.
	 * @return Collection of ProjectPropertyValue - all project properties
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 * @throws HeroException
	 *
	 **/
	public Collection getProperties() throws HeroException {
	log.info(" start by " + mContext.getCallerPrincipal().getName());
		Collection prp = new ArrayList();
		BnProjectPropertyLocal ppl;

		Collection pls = mProject.getBnProperties();
		for (Iterator i = pls.iterator(); i.hasNext();) {
			ppl = (BnProjectPropertyLocal) i.next();
			prp.add(ppl.getBnProjectPropertyValue());
		}
		log.info("End");
		return (prp);
	}

	/**
	 * Get hooks of the project. <strong> This functionality is disabled.
	 * Use {@link #getInterHooks() getInterHook}
	 * method to get the list of beanshell hooks of the project. </strong>
	 * 
	 * @return Collection of BnProjectHooksValue - all project hook
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 * @throws HeroException
	 *
	 **/
	public Collection getHooks() throws HeroException {
			log.info(" start by " + mContext.getCallerPrincipal().getName());
		Collection hooks = new ArrayList();
		BnProjectHookLocal phl;

		Collection hks = mProject.getBnHooks();
		for (Iterator i = hks.iterator(); i.hasNext();) {
			phl = (BnProjectHookLocal) i.next();
			hooks.add(phl.getBnProjectHookValue());
		}
		log.info("End");
		return (hooks);
	}
	

	/**
	 * Get the project name of this instance
	 *
	 * @param instanceName the name of the instance
	 * @return String - the project name
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 * @throws HeroException
	 *
	 **/
	public String getProjectNameOfInstance(String instanceName) throws HeroException {
		log.info("parameters: instanceName=" + instanceName + " start by " + mContext.getCallerPrincipal().getName());
		if (instanceName.matches(".*"+"_instance"+".*")){
			int i = instanceName.indexOf("_instance");
			log.info("End");
			return(instanceName.substring(0,i));
		}
		else {
			 log.info("End");
			 throw new HeroException(instanceName+" is not a project instance...");
		}
	}
	
	/**
	 * Get the parent project of this project
	 *
	 * @return String - the parent project of this project
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 * @throws HeroException
	 *
	 **/
	public String getParent() throws HeroException {
		log.info(" start by " + mContext.getCallerPrincipal().getName());
		log.info("End");
		return(mProject.getParent());
	}

	
	
	// -------------------------------------------------------------------------
	// Struts business methods
	// -------------------------------------------------------------------------

	/**
	 * Get the StrutsNode. Returns node information with a specific java bean structure (useful in
	 * struts web applications).
	 *
	 * @param nodeName the name of the Node
	 * @return BnStrutsNodeValue object - node data.
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 * @throws HeroException
	 *
	 **/
		public StrutsNodeValue getStrutsNode(String nodeName)
			throws HeroException {
			log.info("parameter: nodeName=" + nodeName + " start by " + mContext.getCallerPrincipal().getName());

			BnNodeLocalHome nHome;
			BnNodeLocal n;
			try {
				nHome = hero.interfaces.BnNodeUtil.getLocalHome();
			} catch (javax.naming.NamingException be) {
				log.error(be.getMessage());
				throw new HeroException(be.getMessage());
			}
			try {
				log.info("getStrutsNode: nodeName=" + nodeName);
				n = nHome.findByName(nodeName, mProject.getId());

				StrutsNodeValue snv = new StrutsNodeValue();
				snv.setProjectName(mProject.getName());
				snv.setName(n.getName());
				snv.setCreator(n.getCreator().getName());
				snv.setRole(n.getBnRole().getName());
				snv.setState(
					hero.interfaces.Constants.Nd.nodeStateName[n.getState()]);
				snv.setType(hero.interfaces.Constants.Nd.nodeTypeName[n.getType()]);
				if (n.getDescription() != null)
					snv.setDescription(n.getDescription());
				if (n.getExecutor() != null)
					snv.setExecutor(n.getExecutor().getName());
				snv.setAnticipable(n.getAnticipable());
				if (n.getDeadline() != null)
					snv.setDeadline(n.getDeadline().toString());
				log.info("End");
				return (snv);
			} catch (FinderException fe) {
				log.error("In BnNode does not exist " + fe.getMessage());
				throw new HeroException("In BnNode does not exist");
			}
		}


	/**
	 * Get the project nodes. Returns project nodes data.
	 *
	 * @return Array of StrutsNodeValue - all project nodes
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 * @throws HeroException
	 *
	 **/
	public Object[] getNodes() throws HeroException {
			log.info(" start by " + mContext.getCallerPrincipal().getName());
		//ProjectNodes nb = new ProjectNodes();		
		ArrayList ns = new ArrayList();
		Collection nodes = new ArrayList();

		nodes = mProject.getBnNodes();
		Iterator i = nodes.iterator();
		while (i.hasNext()) {
			BnNodeLocal n = (BnNodeLocal) i.next();
			StrutsNodeValue snv = new StrutsNodeValue();
			snv.setProjectName(mProject.getName());
			snv.setName(n.getName());
			snv.setCreator(n.getCreator().getName());
			snv.setRole(n.getBnRole().getName());
			snv.setState(Constants.Nd.nodeStateName[n.getState()]);
			snv.setType(Constants.Nd.nodeTypeName[n.getType()]);
			if (n.getDescription() != null)
				snv.setDescription(n.getDescription());
			if (n.getExecutor() != null)
				snv.setExecutor(n.getExecutor().getName());
			snv.setAnticipable(n.getAnticipable());
			if (n.getDeadline() != null)
				snv.setDeadline(n.getDeadline().toString());

			ns.add(snv);
		}
		//nb.setNodes(ns.toArray());
		log.info("End");
		return (ns.toArray());
	}

	/**
	 * Get the StrutsNodes. Returns nodes information with a specific java bean structure (useful in
	 * struts web applications).
	 *
	 * @return Collection of StrutsNodeValue - all project nodes
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 * @throws HeroException
	 *
	 **/
	public Collection getStrutsNodes() throws HeroException {
		log.info(" start by " + mContext.getCallerPrincipal().getName());

		//ctor strutsNodes = new Vector();
		ArrayList strutsNodes = new ArrayList();
		Collection nodes = mProject.getBnNodes();
		Iterator i = nodes.iterator();
		while (i.hasNext()) {
			BnNodeLocal n = (BnNodeLocal) i.next();
			StrutsNodeValue snv = new StrutsNodeValue();
			snv.setName(n.getName());
			snv.setCreator(n.getCreator().getName());
			snv.setRole(n.getBnRole().getName());
			snv.setState(
				hero.interfaces.Constants.Nd.nodeStateName[n.getState()]);
			snv.setType(hero.interfaces.Constants.Nd.nodeTypeName[n.getType()]);
			snv.setDescription(n.getDescription());
			if (n.getExecutor() != null)
				snv.setExecutor(n.getExecutor().getName());
			snv.setAnticipable(n.getAnticipable());
			if (n.getDeadline() != null)
				snv.setDeadline(n.getDeadline().toString());
			strutsNodes.add(snv);
		}
		log.info("End");
		return ((Collection) strutsNodes);
	}

	/**
	 * Get the Edges of the Node. Returns node edges information with a specific java bean structure (useful in
	 * struts web applications).
	 *
	 * @param nodeName the name of the Node
	 * @return Collection of StrutsEdgeValue - all node edges
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 * @throws HeroException
	 *
	 **/
	public Collection getStrutsNodeEdges(String nodeName)
		throws HeroException {
		log.info("parameter: nodeName=" + nodeName + " start by " + mContext.getCallerPrincipal().getName());

		BnNodeLocalHome nHome;
		BnNodeLocal mNode;
		BnNodeValue nv;

		//	Vector edges = new Vector();
		ArrayList edges = new ArrayList();

		try {
			nHome = hero.interfaces.BnNodeUtil.getLocalHome();
		} catch (javax.naming.NamingException be) {
			log.error(be.getMessage());
			throw new HeroException(be.getMessage());
		}
		try {
			mNode = nHome.findByName(nodeName, mProject.getId());
		} catch (FinderException fe) {
			log.error("In Node does not exist " + fe.getMessage());
			throw new HeroException("In Node does not exist");
		}
		Collection in = mNode.getInBnEdges();
		Collection out = mNode.getOutBnEdges();

		Iterator iin = in.iterator();
		while (iin.hasNext()) {
			StrutsEdgeValue sev = new StrutsEdgeValue();
			hero.interfaces.BnEdgeLocal actualEdge =(hero.interfaces.BnEdgeLocal) iin.next();
			sev.setState(hero.interfaces.Constants.Ed.edgeStateName[actualEdge.getState()]);
			sev.setInNode(((BnNodeLocal) actualEdge.getInBnNode()).getName());
			sev.setOutNode(((BnNodeLocal) actualEdge.getOutBnNode()).getName());
			edges.add(sev);
		}

		Iterator iout = out.iterator();
		while (iout.hasNext()) {
			StrutsEdgeValue sev = new StrutsEdgeValue();
			BnEdgeLocal actualEdge = (BnEdgeLocal) iout.next();
			sev.setState(hero.interfaces.Constants.Ed.edgeStateName[actualEdge.getState()]);
			sev.setInNode(((BnNodeLocal) actualEdge.getInBnNode()).getName());
			sev.setOutNode(((BnNodeLocal) actualEdge.getOutBnNode()).getName());
			edges.add(sev);
		}
		log.info("End");
		return ((Collection) edges);
	}

	/**
	 * Get the StrutsEdges. Returns project edges information with a specific java bean structure (useful in
	 * struts web applications).
	 *
	 * @return Collection of StrutsEdgeValue - all project edges
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 * @throws HeroException
	 *
	 **/
	public Collection getStrutsEdges() throws HeroException {
		log.info(" start by " + mContext.getCallerPrincipal().getName());

		//	Vector strutsEdges = new Vector();
		ArrayList strutsEdges = new ArrayList();
		Collection edges = mProject.getBnEdges();
		Iterator i = edges.iterator();
		while (i.hasNext()) {
			BnEdgeLocal e = (BnEdgeLocal) i.next();
			StrutsEdgeValue sev = new StrutsEdgeValue();
			sev.setName(e.getName());
			sev.setState(
				hero.interfaces.Constants.Ed.edgeStateName[e.getState()]);
			sev.setInNode(e.getInBnNode().getName());
			sev.setOutNode(e.getOutBnNode().getName());
			strutsEdges.add(sev);
		}
		log.info("End");
		return ((Collection) strutsEdges);
	}

	/**
	 * Import all the nodes, hooks and edges of a project to the current project
	 *
	 * @param projectname the name of the project
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 * @throws HeroException
	 *
	 **/
	public void importProject(String projectName) throws HeroException {
		log.info("parameter: project=" + projectName + " start by " + mContext.getCallerPrincipal().getName());
		importProject(projectName, "");
		log.info("End");
	}
	
     //	-------------------------------------------------------------------------
	 // ejbCreate and initProject business methods
	 // -------------------------------------------------------------------------

	 /**
	  * Import all the nodes, hooks, properties and edges of a project to the current project
	  *
	  * @param projectname the name of the project
	  * @param prefix the prefix of names for the project
	  * @ejb:interface-method view-type="both"
	  * @ejb:transaction type="Required"
	  * @throws HeroException
	  *
	  **/
	 public void importProject(String projectName, String prefix)
		 throws HeroException {
		 log.info("parameters: project=" + projectName + " prefix=" + prefix + " start by " + mContext.getCallerPrincipal().getName());
		 try {
			 BnProjectLocalHome opHome =	hero.interfaces.BnProjectUtil.getLocalHome();
			 BnProjectLocal oldProject = opHome.findByName(projectName);
			 String actualProject = this.getName();

			 // Project Properties (not initialized)
			  Collection props = oldProject.getBnProperties();
			  for (Iterator i = props.iterator(); i.hasNext();) {
				  BnProjectPropertyLocal prop = (BnProjectPropertyLocal) i.next();
				  this.setProperty(prop.getTheKey(),"");
			  }

			 //Project InterHooks
			 Collection hooks = oldProject.getBnInterHooks();
			 for (Iterator i = hooks.iterator(); i.hasNext();) {
				 BnProjectInterHookLocal hk = (BnProjectInterHookLocal) i.next();
				 this.addInterHook(hk.getName(),hk.getEvent(),hk.getType(),hk.getScript());
			 }
			 
			Collection roles = oldProject.getBnRoles();
			for (Iterator i = roles.iterator(); i.hasNext();)
			{
			  BnRoleLocal ro = (BnRoleLocal)i.next();
			  String roleName = ro.getName();
			  // add mapper for the instance
			  if (!containsRole(roleName))
			    this.addRole(roleName, ro.getDescription());
			  if (ro.getBnRoleMapper()!=null)	  
			    this.addRoleMapper(ro.getName(),ro.getBnRoleMapper().getName(),ro.getBnRoleMapper().getType());
			}

			 //	Copy not connected nodes
			 Collection allNodes = oldProject.getBnNodes();
			 Iterator no = allNodes.iterator();
			 while (no.hasNext()) {
				 BnNodeLocal node = (BnNodeLocal) no.next();
				 if (node.getInBnEdges().isEmpty()&& node.getOutBnEdges().isEmpty()) {
				 // New Instance if activity is a subProcess
					 String newNodeName=node.getName();
					 if (node.getType() == hero.interfaces.Constants.Nd.SUB_PROCESS_NODE) {
						 Iterator pr = subProjects.iterator();
						 boolean exit = false;
						 while (pr.hasNext()) {
						 String nodePr = (String) pr.next();
						 if (nodePr.equals(newNodeName)) {
							 newNodeName = actualProject + nodePr;
							 exit = true;
						 }
					 }
					 try {
						 if (!exit) {
						   this.initProject(newNodeName, actualProject + newNodeName);
						   subProjects.add(newNodeName);
						   newNodeName = actualProject + newNodeName;
						   mProject = opHome.findByName(actualProject);
						 }
					 } catch (Exception ce) {
						 log.error(ce.getMessage());
						 ce.printStackTrace();
					 }			
				 }			
					
					 this.addNode(newNodeName, node.getType());
					 if (!node.getAnticipable())
						 this.setNodeTraditional(newNodeName);
					 this.setNodeRole(newNodeName,node.getBnRole().getName());
					 this.setNodeDescription(newNodeName, node.getDescription());

					 //Node InterHooks
					 Collection nodeHooks = node.getBnInterHooks();
					 for (Iterator j = nodeHooks.iterator(); j.hasNext();) {
						 BnNodeInterHookLocal nh = (BnNodeInterHookLocal) j.next();
						 try {
							 this.addNodeInterHook(newNodeName,nh.getName(),nh.getEvent(),nh.getType(),nh.getScript());
						 } catch (Exception ih) {
							 log.error(ih.getMessage());
						 }; // For subProcess Instance consistency
					 }
				 }
			 }
			
			 Collection edges = oldProject.getBnEdges();
			 Iterator e = edges.iterator();
			 while (e.hasNext())
			 {
				 BnEdgeLocal ed = (BnEdgeLocal)e.next();
				 BnNodeLocal nodeIn = ed.getInBnNode();
				 BnNodeLocal nodeOut = ed.getOutBnNode();
				 BnNodeLocal[] nodes = { nodeIn, nodeOut };
				 String newNodeNameIn = null;
				 String newNodeNameOut = null;
				 for (int n = 0; n < nodes.length; n++) {
					 String newNodeName = nodes[n].getName();
					 // New Instance if activity is a subProcess
					 if (nodes[n].getType()== hero.interfaces.Constants.Nd.SUB_PROCESS_NODE){
						 Iterator pr = subProjects.iterator();
						 boolean exit = false;
						 while (pr.hasNext()) {
							 String nodePr = (String) pr.next();
							 if (nodePr.equals(newNodeName)) {
								 newNodeName = actualProject+nodePr;
								 exit = true;
							 }
						 }
						 try {
						 if (!exit) {
							 this.initProject(newNodeName,actualProject+newNodeName);
							 subProjects.add(newNodeName);
							 newNodeName=actualProject+newNodeName;
							 mProject = opHome.findByName(actualProject);
						 }
						 }catch(Exception ce){
							 log.error(ce.getMessage());
							 ce.printStackTrace();
						  }
					 }
					 try {
						 this.addNode(newNodeName, nodes[n].getType());
						 if (!nodes[n].getAnticipable())
							 this.setNodeTraditional(newNodeName);
						 this.setNodeRole(newNodeName,nodes[n].getBnRole().getName());
						 this.setNodeDescription(newNodeName,nodes[n].getDescription());
						 // Node iteration properties
						 Collection nodeProps = oldProject.getBnNode(nodes[n].getName()).getBnProperties();
						 for (Iterator i = nodeProps.iterator(); i.hasNext();) {
							  BnNodePropertyLocal prop = (BnNodePropertyLocal) i.next();
							  if (prop.getTheKey().equals("iterate"))
								 this.setNodeProperty(newNodeName,prop.getTheKey(),prop.getTheValue());
						  }

						 //Node InterHooks
						 BnNodeLocal nd = oldProject.getBnNode(nodes[n].getName());
						 Collection nodeHooks = nd.getBnInterHooks();
						 for (Iterator j = nodeHooks.iterator(); j.hasNext();) {
							 BnNodeInterHookLocal nh = (BnNodeInterHookLocal) j.next();
							 try {
								 this.addNodeInterHook(newNodeName,nh.getName(),nh.getEvent(),nh.getType(),nh.getScript());
							 } catch (Exception ih) {
								 log.error(ih.getMessage());
							 }; // For subProcess Instance consistency
						 }
					 } catch (Exception an) {
						 log.error(an.getMessage());
					 }; // Node already exist
					 if (n == 0)
						 newNodeNameIn = newNodeName;
					 else
						 newNodeNameOut = newNodeName;
				 }

				 String edgeName = this.addEdge(newNodeNameIn, newNodeNameOut);
				 this.setEdgeCondition(edgeName, ed.getCondition());
			 }

		 } catch (FinderException e) {
			 log.error("Cannot find project " + e.getMessage());
			 throw new HeroException("Cannot find project " + e.getMessage());
		 } catch (javax.naming.NamingException ne) {
			 log.error(ne.getMessage());
			 throw new HeroException(ne.getMessage());
		 }
		 log.info("End");
	 }

	 /**
	  * Import Project instance. Import all the nodes, hooks, properties,edges and users of a project instant to the
	  * current project
	  *
	  * @param projectname the name of the project
	  * @param instance the instance name of the project
	  * @ejb:interface-method view-type="both"
	  * @ejb:transaction type="Required"
	  * @throws HeroException
	  *
	  **/
	 public void importInstance(String projectName, String instance)
		 throws HeroException {
		 log.info("parameter: project=" + projectName + " start by " + mContext.getCallerPrincipal().getName());
		 try {
			 BnProjectLocalHome opHome =	hero.interfaces.BnProjectUtil.getLocalHome();
			 BnProjectLocal oldProject = opHome.findByName(projectName);
			
			 BnProjectValue pv = oldProject.getBnProjectValue();
			

			 // Project Properties (not initialized)

			 BnProjectPropertyValue[] props = pv.getBnProperties();
			 Collection insertProp = new ArrayList();
			 int ppIndex = 0;
			 for (ppIndex=0;ppIndex<props.length;ppIndex++)
			 {
				 BnProjectPropertyValue prop = props[ppIndex];
				 this.setProperty(props[ppIndex].getTheKey(),"");
				
			 }

			 BnProjectInterHookValue[] hooks = pv.getBnInterHooks();
			 Collection insertHooks = new ArrayList();
			 int pihIndex = 0;
			 for (pihIndex=0;pihIndex<hooks.length;pihIndex++)
			 {
				 BnProjectInterHookValue hk = hooks[pihIndex];
				 this.addInterHook(hk.getName(),hk.getEvent(),hk.getType(),hk.getScript());
			 }
						 
			BnRoleValue[] roles = pv.getBnRoles();
			int rIndex = 0;
			for (rIndex=0;rIndex<roles.length;rIndex++)
			{
				BnRoleValue ro = roles[rIndex];
				String roleName = ro.getName();
				// add mapper for the instance
				if (!containsRole(roleName))
					this.addRole(roleName, ro.getDescription());
				if (ro.getBnRoleMapper()!=null)	  
				  this.addRoleMapper(ro.getName(),ro.getBnRoleMapper().getName(),ro.getBnRoleMapper().getType());
			}
			
			 // Copy not connected nodes
			
			 BnNodeValue[] allNodes = pv.getBnNodes();
			 //Collection insertNodes = new ArrayList();
			 int nIndex = 0;
			 for (nIndex=0;nIndex<allNodes.length;nIndex++)
			 {
				 Collection insertNodeHooks = new ArrayList();
				 BnNodeValue node = allNodes[nIndex];
				 BnNodeLocal nodeLocal = oldProject.getBnNode(node.getName());
				 if (nodeLocal.getInBnEdges().size() == 0 && nodeLocal.getOutBnEdges().size()==0)
				 {
					 String newNodeName=node.getName();
					 if (node.getType()== hero.interfaces.Constants.Nd.SUB_PROCESS_NODE && !subProjects.contains(node.getName())) {
						 Iterator pr = subProjects.iterator();
						 boolean exit=false;
						 while (pr.hasNext())
						 {
						   String nodePr = (String)pr.next();
						   if (nodePr.matches(newNodeName+"_instance"+".*"))
						   {
							   newNodeName = nodePr;
							   exit = true;
						   }	
						  }
						  if (!exit)
						  {
							 newNodeName = this.instantiateProject(newNodeName);
							 subProjects.add(newNodeName);
							 mProject.setParent(instance);
							 mProject = opHome.findByName(instance);
						  }
					 }
					 try{					
					   this.addNode(newNodeName, node.getType());
					   if (!node.getAnticipable())
						 this.setNodeTraditional(newNodeName);
					   this.setNodeRole(newNodeName,node.getBnRole().getName());
					   this.setNodeDescription(newNodeName,node.getDescription());
					   
					   if (node.getBnNodePerformerAssign()!=null)
						   this.addNodePerformerAssign(node.getName(),node.getBnNodePerformerAssign().getName(),node.getBnNodePerformerAssign().getType(),node.getBnNodePerformerAssign().getPropertyName());
				   
					 }catch(Exception ne){
						 log.error(ne.getMessage());
					 }
					 BnNodeInterHookValue[] nodeHooks = node.getBnInterHooks();
					 int niIndex = 0;
					 for (niIndex=0;niIndex<nodeHooks.length;niIndex++)
					 {
						 BnNodeInterHookValue nh = nodeHooks[niIndex];
						 try {
							 this.addNodeInterHook(newNodeName,nh.getName(),nh.getEvent(),nh.getType(),nh.getScript());
						 } catch (Exception ih) {
							 log.error(ih.getMessage());
						 }; // For subProcess Instance consistency
					 }
				 }
			 }
				
			 // Copy connected nodes and edges
			
			 BnEdgeValue[] edges = pv.getBnEdges();
			
			 int eIndex = 0;
			 for (eIndex=0;eIndex<edges.length;eIndex++)
			 {
				 BnEdgeValue ed = edges[eIndex];
				 BnNodeValue nodeIn = ed.getInBnNode();
				 BnNodeValue nodeOut = ed.getOutBnNode();
				 BnNodeValue[] nodes = { nodeIn, nodeOut };
				 String newNodeNameIn = null;
				 String newNodeNameOut = null;
				 for (int n = 0; n < nodes.length; n++) {
					 String newNodeName = nodes[n].getName();
					 // New Instance if activity is a subProcess
					 if (nodes[n].getType()== hero.interfaces.Constants.Nd.SUB_PROCESS_NODE && !subProjects.contains(nodes[n].getName())) {
						 Iterator pr = subProjects.iterator();
						 boolean exit=false;
						 while (pr.hasNext())
						 {
   							 String nodePr = (String)pr.next();
							 if (nodePr.matches(newNodeName+"_instance"+".*"))
							 {
							   newNodeName = nodePr;
							   exit = true;
							 }	
						 }
						 if (!exit)
						 {
							 newNodeName = this.instantiateProject(newNodeName);
							 subProjects.add(newNodeName);
							 mProject.setParent(instance);
							 mProject = opHome.findByName(instance);
						 }
					 }
					 try{					  	
					   this.addNode(newNodeName, nodes[n].getType());
					   if (!nodes[n].getAnticipable())
						   this.setNodeTraditional(newNodeName);
					   this.setNodeRole(newNodeName,nodes[n].getBnRole().getName());
					   this.setNodeDescription(newNodeName,nodes[n].getDescription());	
					   
					   if (nodes[n].getBnNodePerformerAssign()!=null)
						   this.addNodePerformerAssign(nodes[n].getName(),nodes[n].getBnNodePerformerAssign().getName(),nodes[n].getBnNodePerformerAssign().getType(),nodes[n].getBnNodePerformerAssign().getPropertyName());
			  			 					  
 					   BnNodePropertyValue[] nodeProps = nodes[n].getBnProperties();
					   int nopIndex=0;
					   for (nopIndex=0;nopIndex<nodeProps.length;nopIndex++)
					   {
						 BnNodePropertyValue prop = nodeProps[nopIndex];
						 if (prop.getTheKey().equals("iterate"))
							this.setNodeProperty(newNodeName,prop.getTheKey(),prop.getTheValue());
						 else  	
							this.setNodeProperty(newNodeName,prop.getTheKey(),"");
					   }
					 
					   //Node InterHooks
					   BnNodeInterHookValue[] nodeHooks = nodes[n].getBnInterHooks();
					   int nihIndex=0;
					   for (nihIndex=0;nihIndex<nodeHooks.length;nihIndex++)
					   {
						 BnNodeInterHookValue nh = nodeHooks[nihIndex];
						 try {
						  this.addNodeInterHook(newNodeName,nh.getName(),nh.getEvent(),nh.getType(),nh.getScript());
						 } catch (Exception ih) {
							 log.error(ih.getMessage());
						  };  // For subProcess Instance consistency
					   }
					 }catch(Exception an){
						 log.error(an.getMessage());
					 }; // Node already exist
					 if (n == 0)
						 newNodeNameIn = newNodeName;
					 else
						 newNodeNameOut = newNodeName;
				 }

				 String edgeName = this.addEdge(newNodeNameIn, newNodeNameOut);
				 this.setEdgeCondition(edgeName,ed.getCondition());
			 }
			 //pvInsert.setAddedBnNodes(insertNodes);
			
			 BnUserLightValue[] users = pv.getBnUsers();
			 int uIndex = 0;
			 for (uIndex=0;uIndex<users.length;uIndex++)
			 {
				 BnUserLightValue userP = users[uIndex];
				 try {
					this.addUser(userP.getName());
				 } catch (Exception eUser) {
					 log.error(eUser.getMessage());
				 jms.sendProjectEvent(ADDUSERPROJECT,mProject.getName(),userP.getName());
				 }
				 mProject = oldProject;
				 Collection userRoles = this.getUserRolesInProjectNames(userP.getName());
				 mProject = opHome.findByName(instance);
				 Iterator ir = userRoles.iterator();
				 while (ir.hasNext())
					 this.setUserRole(userP.getName(),(String) ir.next());
			 }

		 } catch (FinderException e) {
			 log.error("Cannot find project " + e.getMessage());
			 throw new HeroException("Cannot find project " + e.getMessage());
		 } catch (javax.naming.NamingException ne) {
			 log.error(ne.getMessage());
			 throw new HeroException(ne.getMessage());
		 } catch (CreateException ce) {
			 log.error("Instanciate SubProcess error " + ce.getMessage());
			 throw new HeroException("Instanciate SubProcess error" + ce.getMessage());
		 }
		 log.info("End");
	 }

	 /**
	  * Init the Project Session Bean. You have to call this method after "create" call. After that
	  * all methods of ProjectSession API are available.
	  *
	  * @param projectName project name 
	  * @ejb:interface-method view-type="both"
	  * @ejb:transaction type="Required"
	  * @throws HeroException
	  **/
	 public void initProject(String projectName) throws HeroException {
		 Context lContext;
		 BnUserLocalHome uhome = null;
		 BnRoleLocalHome rhome = null;
		 log.info("parameter: project=" + projectName + " start by " + mContext.getCallerPrincipal().getName());
		   try {
			 pHome = hero.interfaces.BnProjectUtil.getLocalHome();
			 uhome = hero.interfaces.BnUserUtil.getLocalHome();
			 rhome = hero.interfaces.BnRoleUtil.getLocalHome();
			 this.mProject = pHome.findByName(projectName);
		   } catch (javax.naming.NamingException ne) {
			 log.error(ne.getMessage());
			 throw new HeroException(ne.getMessage());
		   } catch (javax.ejb.FinderException fe) {
			 log.error(fe.getMessage());
			 // The project has to be created
			 try {
				 String creator = mContext.getCallerPrincipal().getName();
				 mProject = pHome.create(projectName, creator);
				 mProject.setInstanceNs(0);
				 mProject.setParent(projectName);
				 BnUserLocal user = uhome.findByName(creator);
				 mProject.getBnUsers().add(user);
				 log.info("Add user " + user.getBnUserLightValue().toString());
				 // role admin is added
				 BnRoleValue rv = new BnRoleValue();
				 rv.setName(hero.interfaces.Constants.ADMIN);
				 rv.setDescription("the admin role");
				 BnRoleLocal role = rhome.create(rv);
				 mProject.getBnRoles().add(role);
				 this.setUserRole(creator, hero.interfaces.Constants.ADMIN);
				 // role InitialRole is added
				 rv = new BnRoleValue();
				 rv.setName(hero.interfaces.Constants.INITIALROLE);
				 rv.setDescription("the initial role for nodes");
				 role = rhome.create(rv);
				 mProject.getBnRoles().add(role);
				 mProject.setCreationDate(new Date());
				 jms.sendProjectEvent(ADDUSERPROJECT, projectName, creator);
			  } catch (FinderException fie) {
				 log.error(fie.getMessage());
				 throw new HeroException(fie.getMessage());
			  } catch (InvalidValueException ie) {
				 log.error(ie.getMessage());
				 throw new HeroException(ie.getMessage());
			  } catch (Exception he) {
				 log.error(he.getMessage());
				 throw new HeroException(he.getMessage());
			  }
			}
			log.info("End");
	 }

	 /**
	  * init the Project Session Bean (clone project).You have to call this method after "create" call. After that
	  * the project is cloned and all methods of ProjectSession API are available.
	  *
	  * @throws HeroException 
	  * @param oldProject the name of the project to clone
	  * @param newProject the name of the project clone.
	  * @ejb:interface-method view-type="both"
	  * @ejb:transaction type="Required"
	  * @throws CreateException
	  **/
	 public void initProject(String oldProject, String newProject)
		 throws CreateException {
		 log.info("parameters: oldProject="+ oldProject+ " newProject="+ newProject + " start by " + mContext.getCallerPrincipal().getName());
		 Context lContext;
		 BnUserLocalHome uhome = null;
		 BnRoleLocalHome rhome = null;
		   try {
			 pHome = hero.interfaces.BnProjectUtil.getLocalHome();
			 uhome = hero.interfaces.BnUserUtil.getLocalHome();
			 rhome = hero.interfaces.BnRoleUtil.getLocalHome();
			 mProject = pHome.findByName(newProject);
			} catch (javax.naming.NamingException ne) {
			 log.error(ne.getMessage());
			 throw new CreateException(ne.getMessage());
			} catch (javax.ejb.FinderException fe) {
			 log.error(fe.getMessage());
			 // The project has to be created
			 try {
				 String creator = mContext.getCallerPrincipal().getName();
				 mProject = pHome.create(newProject, creator);
				 mProject.setInstanceNs(0);
				 mProject.setParent(newProject);
				 BnUserLocal user = uhome.findByName(creator);
				 mProject.getBnUsers().add(user);
				
				 log.info("Add user " + user.getBnUserLightValue().toString());
				 // role admin is added
				 BnRoleValue rv = new BnRoleValue();
				 rv.setName(hero.interfaces.Constants.ADMIN);
				 rv.setDescription("the admin role");
				 BnRoleLocal role = rhome.create(rv);
				 mProject.getBnRoles().add(role);
				 this.setUserRole(creator, hero.interfaces.Constants.ADMIN);
				 // role InitialRole is added
				 rv = new BnRoleValue();
				 rv.setName(hero.interfaces.Constants.INITIALROLE);
				 rv.setDescription("the initial role for nodes");
				 role = rhome.create(rv);
				 mProject.getBnRoles().add(role);
				 importProject(oldProject);
				 mProject.setCreationDate(new Date());

				 jms.sendProjectEvent(ADDUSERPROJECT, newProject, creator);

			   } catch (FinderException fie) {
				 log.error(fie.getMessage());
				 throw new CreateException(fie.getMessage());
			   } catch (InvalidValueException ie) {
				 log.error(ie.getMessage());
				 throw new CreateException(ie.getMessage());
			   } catch (HeroException he) {
				 log.error(he.getMessage());
				 throw new CreateException(he.getMessage());
			   }
		   }
		   log.info("End");
	 }

	 /**
	  * Project instance. You have to call this method after "create" call. After that
	  * you have an instance of the project and all methods of ProjectSession API are available.
	  *
	  * @param project the name of the project to instance
	  * @return String - the name of the project instance
	  * @ejb:interface-method view-type="both"
	  * @ejb:transaction type="Required"
	  * @throws CreateException 
	  **/
	 public String instantiateProject(String project) throws CreateException {
		 log.info("parameter: project=" + project + " start by " + mContext.getCallerPrincipal().getName());
		 Context lContext;
		 BnUserLocalHome uhome = null;
		 BnRoleLocalHome rhome = null;
		 BnProjectLocal projectModel = null;
		 String instance = project;
		 try {
			 pHome = hero.interfaces.BnProjectUtil.getLocalHome();
			 projectModel = pHome.findByName(project);
			 int number = projectModel.getInstanceNs() + 1;
			 instance = project + "_instance" + number;
			 uhome = hero.interfaces.BnUserUtil.getLocalHome();
			 rhome = hero.interfaces.BnRoleUtil.getLocalHome();
			 mProject = pHome.findByName(instance);
		 } catch (javax.naming.NamingException ne) {
			 log.error(ne.getMessage());
			 throw new CreateException(ne.getMessage());
		 } catch (javax.ejb.FinderException fe) {
			 log.error(fe.getMessage());
			 // The project has to be created
			 try {
				 String creator = mContext.getCallerPrincipal().getName();
				 mProject = pHome.create(instance, creator);
				 mProject.setParent(instance);
				 BnUserLocal user = uhome.findByName(creator);
				 mProject.getBnUsers().add(user);
				 log.info("Add user " + user.getBnUserLightValue().toString());
				 // role admin is added
				 BnRoleValue rv = new BnRoleValue();
				 rv.setName(hero.interfaces.Constants.ADMIN);
				 rv.setDescription("the admin role");
				 BnRoleLocal role = rhome.create(rv);
				 mProject.getBnRoles().add(role);
				 this.setUserRole(creator, hero.interfaces.Constants.ADMIN);
				 // role InitialRole is added
				 rv = new BnRoleValue();
				 rv.setName(hero.interfaces.Constants.INITIALROLE);
				 rv.setDescription("the initial role for nodes");
				 role = rhome.create(rv);
				 mProject.getBnRoles().add(role);
				 importInstance(project, instance);
				 projectModel.setInstanceNs(projectModel.getInstanceNs() + 1);
				 mProject.setCreationDate(new Date());

				 // mapper execution
				 this.mapRole();

				 jms.sendProjectEvent(ADDUSERPROJECT, instance, creator);

			 } catch (FinderException fie) {
				 log.error(fie.getMessage());
				 throw new CreateException(fie.getMessage());
			 } catch (InvalidValueException ie) {
				 log.error(ie.getMessage());
				 throw new CreateException(ie.getMessage());
			 } catch (HeroException he) {
				 log.error(he.getMessage());
				 throw new CreateException(he.getMessage());
			 }
		 }
		 log.info("End");
		 thelog.debug("User: +'"+ mContext.getCallerPrincipal().getName() + "' instantiates project: '" + project + "'. '" + instance + "' is created");
		 return (instance);
	 }
	
	/**
	 * Creates the Project Session Bean. This method is the first one to invoke in order to
	 * use ProjectSession API. If the user is not authorized this method throws an exception.
	 *
	 * @throws CreateException 
	 *
	 * @ejb:create-method view-type="both"
	 **/
	public void ejbCreate() throws CreateException {
	}

	/**
	 * Internal Enterprise Java Beans method. 
	**/
	public void ejbPostCreate(String oldProject, String newProject)
		throws CreateException, RemoteException {
	}

	/**
	 * Internal Enterprise Java Beans method. 
	**/
	public void setSessionContext(final javax.ejb.SessionContext context) {
		mContext = context;
		try{
		  JMSServicesSessionLocalHome home = JMSServicesSessionUtil.getLocalHome();
		  this.jms=home.create();
		}catch(Exception e){throw new EJBException(e.getMessage());}
	}

	/**
	 * Internal Enterprise Java Beans method. 
	**/

	public void ejbRemove() {
	}

	/**
	 * Internal Enterprise Java Beans method. 
	**/

	public void ejbActivate() {
	}

	/**
	 * Internal Enterprise Java Beans method. 
	**/

	public void ejbPassivate() {
	}

	// Utility methods

	private boolean checkDynamics(String out) throws HeroException {
		BnNodeLocal nodeout = mProject.getBnNode(out);
		if ((nodeout.getState() == hero.interfaces.Constants.Nd.INITIAL)
			|| (nodeout.getState() == hero.interfaces.Constants.Nd.READY)
			|| (nodeout.getState() == hero.interfaces.Constants.Nd.ANTICIPABLE))
			return true;
		return false;
	}
	
	 private void mapRole() throws  HeroException {
	 try {	 
		 String creator = mContext.getCallerPrincipal().getName();
		 Collection mappers = new ArrayList();
		 Collection roles = mProject.getBnRoles();
		 BnRoleMapperLocal mapper = null;
		 Iterator it  = roles.iterator();
				 
		it  = roles.iterator();
		 while (it.hasNext()){
			 BnRoleLocal role = (BnRoleLocal) it.next();
			 mapper = role.getBnRoleMapper();
			 if (mapper != null){
				hero.mapper.Mapper h = hero.mapper.Mapper.make(mapper.getName(), mapper.getType());
				h.execute(this, mapper.getType(), role, creator);
			 }
		 }
	 } catch (HeroException he) {
		 throw new HeroException(he.getMessage());
	 }		 

	 }
	 
	private void performAssign() throws  HeroException {
		 // logs trace
		log.debug("perform assign ");
	   try {	 
		String creator = mContext.getCallerPrincipal().getName();
		Collection performers = new ArrayList();
		Collection nodes = mProject.getBnNodes();
		BnNodePerformerAssignLocal performerAssign = null;
		Iterator it  = nodes.iterator();
				 
	   it  = nodes.iterator();
		while (it.hasNext()){
			BnNodeLocal node = (BnNodeLocal) it.next();
			performerAssign = node.getBnNodePerformerAssign();
			if (performerAssign != null){
			   hero.performerAssign.PerformerAssign h = hero.performerAssign.PerformerAssign.make(performerAssign.getName(), performerAssign.getType());
			   h.execute(this, performerAssign.getType(), node, creator);
			}
		}
	} catch (HeroException he) {
		throw new HeroException(he.getMessage());
	}		 

	}

	private boolean checkStructuralConstraints(String in, String out) throws HeroException {
		visited.add(in);
		BnNodeLocal nodeout = mProject.getBnNode(out);
		if (visited.contains(out))
		  return false;
		Collection edges = nodeout.getOutBnEdges();
		Iterator i = edges.iterator();
		while (i.hasNext())
		{
		  BnEdgeLocal edge = (BnEdgeLocal)i.next();
		  if (!visited.contains(edge.getInBnNode().getName()) && ! checkStructuralConstraints(edge.getInBnNode().getName(),edge.getOutBnNode().getName()))
		  	return false;
		}
		
		return true;
	}

	private boolean checkDuplicateEdge(String in, String out)
		throws HeroException {
		Collection edges = mProject.getBnEdges();
		Iterator i = edges.iterator();
		while (i.hasNext()) {
			BnEdgeLocal edge = (BnEdgeLocal) i.next();
			if ((edge.getInBnNode().getName()).equals(in)
				&& (edge.getOutBnNode().getName()).equals(out))
				return false;
		}
		return true;
	}

	private boolean checkDuplicateAgentEdge(String in, String out)
		throws HeroException {
		Collection edges = mProject.getBnAgentEdges();
		Iterator i = edges.iterator();
		while (i.hasNext()) {
			BnAgentEdgeLocal edge = (BnAgentEdgeLocal) i.next();
			if ((edge.getInBnAgent().getName()).equals(in)
				&& (edge.getOutBnNode().getName()).equals(out))
				return false;
		}
		return true;
	}

	/**
	* Test if the project contains this user
	* @param userName The name of the user to associate to the project
	* @return boolean - true is the user belongs to the project
	*
	* @ejb:interface-method view-type="both"
	* @ejb:transaction type="Supports"
	* @throws HeroException
	*
	**/
	public boolean containsUser(String userName) throws HeroException {
		log.debug("containsUser : name=" + userName);
		BnUserLightValue[] uv = mProject.getBnProjectValue().getBnUsers();
			for (int i = 0; i < uv.length; i++) {
				if (uv[i].getName().equalsIgnoreCase(userName))
					return true;
			}
			return false;
		}

	private boolean containsRole(String roleName) {
		BnRoleValue[] rv = mProject.getBnProjectValue().getBnRoles();
		for (int i = 0; i < rv.length; i++) {
			if (rv[i].getName().equalsIgnoreCase(roleName))
				return true;
		}
		return false;
	}

	private void removeOldNotification(String name) throws HeroException {
		try {
			ObjectName timer = new ObjectName("bonita.jmx:service=timer");
			List list = MBeanServerFactory.findMBeanServer(null);
			MBeanServer server = (MBeanServer) list.iterator().next();

			server.invoke(timer,"removeNotifications",new Object[] { name },new String[] { "".getClass().getName()});
		} catch (javax.management.InstanceNotFoundException ex) {
		} catch (Exception e) {
			throw new HeroException(e.getMessage());
		}
	}

}
